@@ -2,7 +2,7 @@
 /* file aritools.c
 
 ARIBAS interpreter for Arithmetic
-Copyright (C) 1996 O.Forster
+Copyright (C) 1996/2003 O.Forster
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ Address of the author
 	Theresienstr. 39
 	D-80333 Muenchen, Germany
 
-Email	forster@rz.mathematik.uni-muenchen.de
+Email	forster@mathematik.uni-muenchen.de
 */
 /****************************************************************/
 /*
@@ -44,9 +44,12 @@ Email	forster@rz.mathematik.uni-muenchen.de
 /*-------------------------------------------------------------------*/
 /*
 ** date of last change
-** 95-10-29:	modification of function biggcd
-** 97-02-25:	more M_3264 support in multbig, divbig and modbig
-** 99-06-03:    power for exponents a >= 2**16; return to old version of modbig
+** 1995-10-29:	modification of function biggcd
+** 1997-02-25:	more M_3264 support in multbig, divbig and modbig
+** 1999-06-03:  power for exponents a >= 2**16; return to old version of modbig
+** 2002-04-20:  modmultbig, addsarr
+** 2003-06-16:  modnegbig
+** 2003-11-07:  returning to old version of divbig (new version was buggy)
 */
 
 #include "common.h"
@@ -56,13 +59,18 @@ PUBLIC int lshiftarr	_((word2 *x, int n, long sh));
 PUBLIC int addarr	_((word2 *x, int n, word2 *y, int m));
 PUBLIC int subarr	_((word2 *x, int n, word2 *y, int m));
 PUBLIC int sub1arr	_((word2 *x, int n, word2 *y, int m));
+PUBLIC int addsarr  _((word2 *x, int n, int sign1,
+                word2 *y, int m, int sing2, int *psign));
 PUBLIC int multbig	_((word2 *x, int n, word2 *y, int m, word2 *z,
-			   word2 *hilf));
+                word2 *hilf));
 PUBLIC int divbig	_((word2 *x, int n, word2 *y, int m, word2 *quot,
-			  int *rlenptr, word2 *hilf));
+                int *rlenptr, word2 *hilf));
 PUBLIC int modbig	_((word2 *x, int n, word2 *y, int m, word2 *hilf));
+PUBLIC int modnegbig	_((word2 *x, int n, word2 *y, int m, word2 *hilf));
+PUBLIC int modmultbig   _((word2 *xx, int xlen, word2 *yy, int ylen,
+                word2 *mm, int mlen, word2 *zz, word2 *hilf));
 PUBLIC int multfix	_((int prec, word2 *x, int n, word2 *y, int m,
-			   word2 *z, word2 *hilf));
+                word2 *z, word2 *hilf));
 PUBLIC int divfix	_((int prec, word2 *x, int n, word2 *y, int m,
 			   word2 *z, word2 *hilf));
 PUBLIC unsigned shortgcd  _((unsigned x, unsigned y));
@@ -220,6 +228,60 @@ int n, m;
 }
 /*-------------------------------------------------------------------*/
 /*
+** Addition of signed bigintegers
+** non-negative numbers: sign = 0;
+** negative numbers:     sign = MINUSBYTE;
+**
+** array x is overwritten with the result; must be long enough
+** sign of result in *psign
+*/
+PUBLIC int addsarr(x,n,sign1,y,m,sign2,psign)
+word2 *x, *y;
+int sign1, sign2;
+int *psign;
+{
+    int k;
+    int cmp;
+
+    if(sign1 == sign2) {
+        *psign = sign1;
+	    if(n < m) {
+		    cpyarr(y+n,m-n,x+n);
+		    k = m;			/* swap m, n */
+		    m = n;
+		    n = k;
+	    }
+	    if(sumarr(x,m,y))
+		    n = m + incarr(x+m,n-m,1);
+	    return(n);
+    }
+    /* else */
+    cmp = cmparr(x,n,y,m);
+    if(cmp > 0) {
+        *psign = sign1;
+    	if(diffarr(x,m,y))
+	    	n = m + decarr(x+m,n-m,1);
+    	while(n > 0 && x[n-1] == 0)
+	    	n--;
+    	return(n);
+    }
+    else if(cmp < 0) {
+        *psign = sign2;
+    	if(n < m)
+	    	cpyarr(y+n,m-n,x+n);
+    	if(diff1arr(x,n,y))
+	    	m = n + decarr(x+n,m-n,1);
+    	while(m > 0 && x[m-1] == 0)
+	    	m--;
+    	return(m);
+    }
+    else {
+        *psign = 0;
+        return 0;
+    }
+}
+/*-------------------------------------------------------------------*/
+/*
 ** z := (x,n) * (y,m)
 ** Rueckgabewert Laenge von z
 ** hilf ist Platz fuer Hilfsvariable;
@@ -282,91 +344,6 @@ int n, m;
 #endif
 }
 /*-------------------------------------------------------------------*/
-#ifdef OLDVERSION
-/*
-** quot = (x,n) / (y,m); Voraussetzung (y,m) != null
-** Arbeitet destruktiv auf x; x wird rest, seine Laenge in *rlenptr
-** hilf ist Platz fuer Hilfsvariable; muss mindestens m+1 lang sein
-** Funktioniert fuer Bestimmung des Restes auch, falls
-** Platz quot und Platz hilf gleich sind und der Quotient
-** nicht interessiert
-*/
-PUBLIC int div_big(x,n,y,m,quot,rlenptr,hilf)
-word2 *x, *y, *quot, *hilf;
-int n, m, *rlenptr;
-{
-	word4 u, v;
-	word2 a;
-	int k, quotlen, hilflen;
-	int cmp, b, b1;
-
-	if(!m) {	/* Division durch 0 ohne Fehlermeldung */
-		*rlenptr = 0;
-		return(0);
-	}
-	if(m == 1) {
-		quotlen = divarr(x,n,(unsigned)y[0],&a);
-		cpyarr(x,quotlen,quot);
-		*x = a;
-		*rlenptr = (a ? 1 : 0);
-		return(quotlen);
-	}
-#ifdef M_3264
-	if(m == 2) {
-		u = big2long(y,m);
-		quotlen = div4arr(x,n,u,&v);
-		cpyarr(x,quotlen,quot);
-		*rlenptr = long2big(v,x);
-		return(quotlen);
-	}
-#endif
-	k = n - m;
-	if(k >= 0)
-		cmp = cmparr(x+k,n-k,y,m);
-	if(k < 0 || (k == 0 && cmp < 0)) {
-		*rlenptr = n;
-		return(0);
-	}
-	else if(k > 0 && cmp < 0)
-		k--;
-
-	b = bitlen(y[m-1]);
-	b1 = 16 - b;
-	v = (y[m-1] << b1) + (y[m-2] >> b);
-	u = 0xFFFFFFFF;
-	u /= (v + 1);
-
-	k++;
-	x += k;
-	n -= k;
-	quot += k;
-	quotlen = k;
-	while(--k >= 0) {
-		--x;
-		if(n || *x)
-			n++;
-		if(n >= m) {
-			v = (x[m-1] >> b);
-			if(n > m)	/* dann n = m+1 */
-				v += (x[m] << b1);
-			a = (u*v) >> 16;
-			hilflen = multarr(y,m,a,hilf);
-			n = subarr(x,n,hilf,hilflen);
-			while(cmparr(x,n,y,m) >= 0) {
-				a++;
-				n = subarr(x,n,y,m);
-			}
-		}
-		else
-			a = 0;
-		*--quot = a;
-	}
-	*rlenptr = n;
-	return(quotlen);
-}
-#endif	/* OLDVERSION */
-/*-------------------------------------------------------------------*/
-/* reinstalled OLDVERSION, 99-06-03, O.F. */
 /*
 ** Ersetzt (x,n) destruktiv durch (x,n) mod (y,m)
 */
@@ -431,6 +408,127 @@ int n, m;
 }
 /*-------------------------------------------------------------------*/
 /*
+** Ersetzt (x,n) destruktiv durch -(x,n) mod (y,m)
+*/
+PUBLIC int modnegbig(x,n,y,m,hilf)
+word2 *x, *y, *hilf;
+int n, m;
+{
+    int len, cmp;
+
+    cmp = cmparr(y,m,x,n);
+    if(cmp == 0)
+        n = 0;
+    else if(cmp < 0)
+        n = modbig(x,n,y,m,hilf);
+    if(n > 0)
+        return sub1arr(x,n,y,m);
+    else
+        return 0;
+}
+/*-------------------------------------------------------------------*/
+/*
+** (zz,ret) = (xx,xlen)*(yy,ylen) mod (mm,mlen)
+** TODO: optimize!
+*/
+PUBLIC int modmultbig(xx,xlen,yy,ylen,mm,mlen,zz,hilf)
+word2 *xx,*yy,*mm,*zz,*hilf;
+int xlen, ylen, mlen;
+{
+    int len;
+
+    len = multbig(xx,xlen,yy,ylen,zz,hilf);
+    len = modbig(zz,len,mm,mlen,hilf);
+    return len;
+}
+/*-------------------------------------------------------------------*/
+/* reinstalled old version 2003-11-08 */
+#if 1
+/*
+** quot = (x,n) / (y,m); Voraussetzung (y,m) != null
+** Arbeitet destruktiv auf x; x wird rest, seine Laenge in *rlenptr
+** hilf ist Platz fuer Hilfsvariable; muss mindestens m+1 lang sein
+** Funktioniert fuer Bestimmung des Restes auch, falls
+** Platz quot und Platz hilf gleich sind und der Quotient
+** nicht interessiert
+*/
+PUBLIC int divbig(x,n,y,m,quot,rlenptr,hilf)
+word2 *x, *y, *quot, *hilf;
+int n, m, *rlenptr;
+{
+	word4 u, v;
+	word2 a;
+	int k, quotlen, hilflen;
+	int cmp, b, b1;
+
+	if(!m) {	/* Division durch 0 ohne Fehlermeldung */
+		*rlenptr = 0;
+		return(0);
+	}
+	if(m == 1) {
+		quotlen = divarr(x,n,(unsigned)y[0],&a);
+		cpyarr(x,quotlen,quot);
+		*x = a;
+		*rlenptr = (a ? 1 : 0);
+		return(quotlen);
+	}
+#ifdef M_3264
+	if(m == 2) {
+		u = big2long(y,m);
+		quotlen = div4arr(x,n,u,&v);
+		cpyarr(x,quotlen,quot);
+		*rlenptr = long2big(v,x);
+		return(quotlen);
+	}
+#endif
+	k = n - m;
+	if(k >= 0)
+		cmp = cmparr(x+k,n-k,y,m);
+	if(k < 0 || (k == 0 && cmp < 0)) {
+		*rlenptr = n;
+		return(0);
+	}
+	else if(k > 0 && cmp < 0)
+		k--;
+
+	b = bitlen(y[m-1]);
+	b1 = 16 - b;
+	v = (y[m-1] << b1) + (y[m-2] >> b);
+	u = 0xFFFFFFFF;
+	u /= (v + 1);
+
+	k++;
+	x += k;
+	n -= k;
+	quot += k;
+	quotlen = k;
+	while(--k >= 0) {
+		--x;
+		if(n || *x)
+			n++;
+		if(n >= m) {
+			v = (x[m-1] >> b);
+			if(n > m)	/* dann n = m+1 */
+				v += (x[m] << b1);
+			a = (u*v) >> 16;
+			hilflen = multarr(y,m,a,hilf);
+			n = subarr(x,n,hilf,hilflen);
+			while(cmparr(x,n,y,m) >= 0) {
+				a++;
+				n = subarr(x,n,y,m);
+			}
+		}
+		else
+			a = 0;
+		*--quot = a;
+	}
+	*rlenptr = n;
+	return(quotlen);
+}
+#endif
+/*-------------------------------------------------------------------*/
+#ifdef BUGGYVERSION
+/*
 ** quot = (x,n) / (y,m); Voraussetzung (y,m) != null
 ** Arbeitet destruktiv auf x; x wird rest, seine Laenge in *rlenptr
 ** hilf ist Platz fuer Hilfsvariable; muss mindestens m+1 lang sein
@@ -486,8 +584,10 @@ int n, m, *rlenptr;
 	b = bitlen(y[m-1]);
 	b1 = 16 - b;
 
-	v = y[m-1]; v <<= 16; v += y[m-2]; v <<= b1; v += (y[m-3] >> b);
-	if(v < 0xFFFFFFFF) 
+	v = y[m-1]; v <<= 16;
+    v += y[m-2]; v <<= b1;
+    v += (y[m-3] >> b);
+	if(v < 0xFFFFFFFF)
 	    v++;
 	ww[3] = 0x7FFF; ww[2] = 0xFFFF; ww[1] = 0xFFFF;
 	/* don't care for ww[0] */
@@ -504,7 +604,7 @@ int n, m, *rlenptr;
 		nu = n - mu;
 		cpyarr(x+mu,nu,ww);
 		nu = mult4arr(ww,nu,v,ww);
-		nu = shrarr(ww+2,nu-2,b);				 
+		nu = shrarr(ww+2,nu-2,b);
 		u = big2long(ww+2,nu);
 		hilflen = mult4arr(y,m,u,hilf);
 		n = subarr(x+kappa,n-kappa,hilf,hilflen);
@@ -517,7 +617,7 @@ int n, m, *rlenptr;
 		if(n == 0) {
 		    n = kappa;
 		    while(n > 0 && x[n-1])
-			n--;
+                n--;
 		}
 		else
 		    n += kappa;
@@ -577,123 +677,7 @@ int n, m, *rlenptr;
 	return(quotlen);
 #endif
 }
-/*-------------------------------------------------------------------*/
-#ifdef BUGGYVERSION
-/*
-** Ersetzt (x,n) destruktiv durch (x,n) mod (y,m)
-*/
-PUBLIC int mod_big(x,n,y,m,hilf)
-word2 *x, *y, *hilf;
-int n, m;
-{
-	word4 u, v;
-	unsigned a;
-	int cmp, b, b1;
-	int k, hilflen;
-#ifdef M_3265
-	int kappa, nu, mu;
-	word2 ww[6];
-#endif
-
-	if(!m)		/* Division durch 0 ohne Fehlermeldung */
-		return(0);
-	if(m == 1) {
-		a = y[0];
-		a = modarr(x,n,a);
-		*x = a;
-		return(a ? 1 : 0);
-	}
-#ifdef M_3264
-	if(m == 2) {
-		u = big2long(y,m);
-		v = mod4arr(x,n,u);
-		k = long2big(v,x);
-		return(k);
-	}
-
-	/* else if(m >= 3) */
-	k = n - m;
-	if(k & 1) k++;	    /* k must be even */
-	if(k >= 0)
-		cmp = cmparr(x+k,n-k,y,m);
-	if(k < 0 || (k == 0 && cmp < 0))
-		return(n);		/* (y,m) > (x,n) */
-	else if(cmp >= 0)
-		k += 2;
-	/* now k >= 2 and (x+k,n-k) < (y,m) */
-	b = bitlen(y[m-1]);
-	b1 = 16 - b;
-
-	v = y[m-1]; v <<= 16; v += y[m-2]; v <<= b1; v += (y[m-3] >> b);
-	if(v < 0xFFFFFFFF)
-	    v++;
-	ww[3] = 0x7FFF; ww[2] = 0xFFFF; ww[1] = 0xFFFF;
-	nu = div4arr(ww,4,v,&u);
-	v = big2long(ww,nu);
-
-	b--;
-	for(kappa = k-2; kappa >= 0; kappa -= 2) {
-		mu = m + kappa - 1;
-		if(n <= mu) {
-		    continue;
-		}
-		nu = n - mu;
-		cpyarr(x+mu,nu,ww);
-		nu = mult4arr(ww,nu,v,ww);
-		nu = shrarr(ww+2,nu-2,b);
-		u = big2long(ww+2,nu);
-		hilflen = mult4arr(y,m,u,hilf);
-		n = subarr(x+kappa,n-kappa,hilf,hilflen);
-		while(cmparr(x+kappa,n,y,m) >= 0) {
-		    n = subarr(x+kappa,n,y,m);
-		}
-		if(n == 0) {
-		    n = kappa;
-		    while(n > 0 && x[n-1])
-			n--;
-		}
-		else
-		    n += kappa;
-	}
-	return(n);
-#else
-	k = n - m;
-	if(k >= 0)
-		cmp = cmparr(x+k,n-k,y,m);
-	if(k < 0 || (k == 0 && cmp < 0))
-		return(n);		/* (y,m) > (x,n) */
-	else if(k > 0 && cmp < 0)
-		k--;
-
-	b = bitlen(y[m-1]);
-	b1 = 16 - b;
-	v = (y[m-1] << b1) + (y[m-2] >> b);
-	u = 0xFFFFFFFF;
-	u /= (v + 1);
-
-	k++;
-	x += k;
-	n -= k;
-	while(--k >= 0) {
-		--x;
-		if(n || *x)
-			n++;
-		if(n >= m) {
-			v = (x[m-1] >> b);
-			if(n > m)	/* dann n = m+1 */
-				v += (x[m] << b1);
-			a = (u*v) >> 16;
-			hilflen = multarr(y,m,a,hilf);
-			n = subarr(x,n,hilf,hilflen);
-			while(cmparr(x,n,y,m) >= 0)
-				n = subarr(x,n,y,m);
-		}
-	}
-	return(n);
-#endif
-}
-#endif /* BUGGYVERSION */
-/*-------------------------------------------------------------------*/
+#endif BUGGYVERSION
 /*-------------------------------------------------------------------*/
 /*
 ** Berechnet Produkt von (x,n)*(2^16)^-prec mit (y,m)*(2^16)^-prec
@@ -802,13 +786,13 @@ unsigned a;
 
 	cpyarr(x,n,p);
 	plen = n;
-        if((b = (a >> 16))) {
-                mask = 0x8000;
-                mask <<= bitlen(b);
-        }
-        else {
-                mask = 1 << (bitlen(a)-1);
-        }
+    if((b = (a >> 16))) {
+        mask = 0x8000;
+        mask <<= bitlen(b);
+    }
+    else {
+        mask = 1 << (bitlen(a)-1);
+    }
 	while(mask >>= 1) {
 		plen = multbig(p,plen,p,plen,temp,hilf);
 		if(a & mask) {
