@@ -2,7 +2,7 @@
 /* file common.h
 
 ARIBAS interpreter for Arithmetic
-Copyright (C) 1996 O.Forster
+Copyright (C) 1996-2002 O.Forster
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ Address of the author
 	Theresienstr. 39
 	D-80333 Muenchen, Germany
 
-Email	forster@rz.mathematik.uni-muenchen.de
+Email	forster@mathematik.uni-muenchen.de
 WWW	http://www.mathematik.uni-muenchen.de/~forster
 
 The latest version of ARIBAS can be obtained by anonymous ftp from
@@ -43,10 +43,12 @@ directory
 ** by more than one C-file
 **
 ** date of last change
-** 97-02-11	moved defn of ARIBUFSIZE to alloc.c
-** 97-04-13	reorg (newintsym)
-** 97-07-04	new #define READLNINPUT
-** 97-11-08	some defines for DjGPP changed
+** 1997-02-11	moved defn of ARIBUFSIZE to alloc.c
+** 1997-04-13	reorg (newintsym)
+** 1997-07-04	new #define READLNINPUT
+** 1997-11-08	some defines for DjGPP changed
+** 2001-03-30	Win32GUI, genWinGUI
+** 2002-03-27   WORKnpush, VECSTRUCTPTR
 */
 
 #include <stdio.h>
@@ -70,21 +72,28 @@ directory
 
 #define Win32CON
 
-
-
 #define DjGPP
 
 #define UNiX
 
+#define UNiX64
+
 #define SCOUNiX
 
 #define LiNUX
+#define GtK	
 ************/
 
 
 
 /*-----------------------------------------------------------------*/
 
+#ifdef UNiX64
+#define ARCHITEC	"UNiX64"
+#define genUNiX
+#define ALIGN8
+#endif
+
 #ifdef UNiX
 #define ARCHITEC	"UNIX"
 #define genUNiX
@@ -97,7 +106,12 @@ directory
 #endif
 
 #ifdef LiNUX
+#ifdef GtK
+#define genWinGUI
+#define ARCHITEC	"LINUX-GTK"
+#else
 #define ARCHITEC	"LINUX386"
+#endif
 #define genUNiX
 #define M_3264
 #endif
@@ -111,6 +125,7 @@ directory
 #ifdef Win32GUI
 #define ARCHITEC    "Win32GUI"
 #define MsWIN32
+#define genWinGUI
 #endif
 
 #ifdef Win32CON
@@ -160,6 +175,10 @@ directory
 #define M_SMALL
 #endif
 
+#ifdef M_LARGE
+#define FPREC_HIGH
+#endif
+
 /*-----------------------------------------------------------------*/
 #ifdef PROTO
 #define _(x)	x
@@ -200,13 +219,17 @@ directory
 #define MAXCOLS		  80	/* max no. of columns on the screen */
 #define ARGCMAX		  64	/* maximal length of ARGV */
 
-#ifdef genUNiX
-#define IOBUFSIZE      1024	/* size of output buffer */
+#ifdef M_LARGE
+#define IOBUFSIZE  1024	/* size of output buffer */
 #define MAXPFADLEN	256	/* avoid name collision with MAXPATHLEN */
-#define SEPPATH		':'
 #else
 #define IOBUFSIZE	256	/* size of output buffer */
 #define MAXPFADLEN	128
+#endif
+
+#ifdef genUNiX
+#define SEPPATH		':'
+#else
 #define SEPPATH		';'
 #endif
 #ifdef UNiXorGCC
@@ -215,16 +238,16 @@ directory
 #define SEPDIR		'\\'
 #endif
 #ifdef genUNiX
-#define SEP_DIR         "/"
+#define SEP_DIR		"/"
 #endif
 #ifdef DOSorTOS
-#define SEP_DIR         "\\"
+#define SEP_DIR		"\\"
 #endif
 #ifdef DjGPP
-#define SEP_DIR         "/\\"
+#define SEP_DIR		"/\\"
 #endif
 #ifndef SEP_DIR
-#define SEP_DIR         "\\/"
+#define SEP_DIR		"\\/"
 #endif
 /*-----------------------------------------------------------------*/
 /* values for flag in symbol structure */
@@ -298,8 +321,11 @@ directory
 
 #define fBOOL		 57
 #define fCHARACTER	 59
+#define fINTTYPE0    60
+#define fGF2NINT     60
 #define fFIXNUM		 61
 #define fBIGNUM		 62
+#define fINTTYPE1    62
 #define fFLTOBJ		128
 #define fHUGEFLOAT  (fFLTOBJ + HUGEFLTBIT)
 
@@ -308,7 +334,7 @@ directory
 #define FLTZEROBIT    0x01	/* for floats = 0 */
 #define HUGEFLTBIT    0x40	/* huge floats */
 #define HUGEMASK      0x7F
-#define FSIGNBIT      0x80  /* sign bit in signum of floats */
+#define FSIGNBIT      0x80	/* sign bit in signum of floats */
 #define GCMARK	      0xFF	/* used during garbage collection */
 #define MINUSBYTE     0xFF	/* sign of negative numbers */
 
@@ -317,23 +343,26 @@ directory
 #define OUTSTREAM	2	/* output stream bit */
 #define IOMASK		3
 #define APPEND		8
-#define BINARY	       16
+#define BINARY	   16
 #define aTEXT		0	/* binary bit not set */
             /* avoid nameclash with TEXT in windows header */
-#define DEVICE	       32	/* console, printer */
+#define DEVICE		32	/* console, printer */
 #define NOSTREAM	0	/* unconnected stream */
 
 
 /* values used for reading and printing */
-#define EOL		'\n'
+#define EOL			'\n'
 #define FORMFEED	'\014'
 #define TABESC		'\036'	/* escape char for compression */
+#define ZESC		'\177'
 
 /* tokens for parser */
 
-#define EOFTOK	       -1      /* end-of-file token */
+#define EOFTOK	   -1      /* end-of-file token */
 #define EOLTOK		0      /* end-of-line token */
 
+#define Z1TOK	-101
+
 #define LPARENTOK	10	/* ( */
 #define RPARENTOK	11	/* ) */
 #define LBRACKTOK	12	/* [ */
@@ -382,19 +411,21 @@ directory
 #define CHARTOK	       2020	/* character token */
 #define INUMTOK	       2030	/* integer number token */
 #define FLOATTOK       2040
+#define GF2NTOK        2045 /* gf2n_int token */
 #define STRINGTOK      2050	/* string token */
 #define BSTRINGTOK     2052	/* byte_string token */
 #define SYMBOLTOK      2060	/* symbol token */
 #define VECTORTOK      2070	/* vector token */
 
 /* Lvals */
-#define vUNBOUND	  0
-#define vBOUND		  1
-#define vCONST		  2
-#define vARRELE		 10
-#define vSUBARRAY	 20
-#define vRECFIELD	 30
-#define vPOINTREF	 40
+#define vUNBOUND	 0
+#define vBOUND		 1
+#define vCONST		 2
+#define vVECTOR	    10
+#define vARRELE		11
+#define vSUBARRAY	12
+#define vRECFIELD	20
+#define vPOINTREF	30
 
 /* defines for diverse return values */
 #define EXITREQ		-1	/* possible return value of loadaux */
@@ -415,22 +446,28 @@ directory
 #define READLNINPUT	4
 
 /*-----------------------------------------------------------------*/
-
+#ifdef M_LARGE
+typedef int int4;		/* 4-byte integer */
+typedef unsigned int word4;
+#else
 typedef long int4;		/* 4-byte integer */
-typedef short int2;		/* 2-byte integer */
 typedef unsigned long word4;
+#endif
+typedef short int2;		/* 2-byte integer */
 typedef unsigned short word2;
 typedef unsigned char byte;
 
 typedef word4	truc;
+typedef void    *wtruc;
+
 typedef truc	*trucptr;
 
 typedef truc	(* funptr)  _((void));
 typedef truc	(* funptr1) _((int k));
-typedef int	(* ifun0)   _((void));
-typedef int	(* ifun)    _((int x));
-typedef int	(* ifunaa)  _((word2 *arr1, int n1, word2 *arr2, int n2));
-typedef int	(* ifuntt)  _((truc *ptr1, truc *ptr2));
+typedef int	    (* ifun0)   _((void));
+typedef int	    (* ifun)    _((int x));
+typedef int	    (* ifunaa)  _((word2 *arr1, int n1, word2 *arr2, int n2));
+typedef int	    (* ifuntt)  _((truc *ptr1, truc *ptr2));
 
 typedef struct {
 	byte	b0;
@@ -449,24 +486,30 @@ typedef union {
 	packet	pp;
 } variant;
 
+typedef union {
+    truc t;
+    wtruc w;
+} wvariant;
+
 struct symbol {		/* symbol structure */
-	truc	ident;
-	truc	bind;		/* symbol binding */
-	char	*name;		/* symbol name */
-	truc	*link;		/* link to next symbol */
-	variant cc;		/* information for syntax checking */
+	truc		ident;
+	variant 	cc;		/* information for syntax checking */
+	wvariant	bind;		/* symbol binding */
+	char		*name;		/* symbol name */
+	truc		*link;		/* link to next symbol */
 };
-#define OFFSETbind	4
-#define OFFSETname	8
-#define OFFSETlink     12
-#define OFFSETcc       16
-#define OFFSETcc1      18
+#define OFFSETcc        4
+#define OFFSETcc1       6
+#define OFFSETbind	    8
+#define OFFSETname	   (OFFSETbind + sizeof(wtruc))
+#define OFFSETlink     (OFFSETbind + 2*sizeof(wtruc))
 #define SIZEOFSYMBOL	(sizeof(struct symbol)/sizeof(truc))
 
 struct intsymbol {	/* internal symbol structure */
-	truc	ident;
-	truc	bind;		/* symbol binding */
-	char	*name;		/* symbol name */
+	truc		ident;
+	variant 	cc;
+	wvariant	bind;		/* symbol binding */
+	char		*name;		/* symbol name */
 };
 #define SIZEOFINTSYMBOL	   (sizeof(struct intsymbol)/sizeof(truc))
 
@@ -477,12 +520,12 @@ struct floatcell {	/* float */
 	word2	digi0;
 	word2	digi1;
 };
-#define OFFSETexpo	2
+#define OFFSETexpo		2
 #define OFFSETflodig	4
 #define SIZEOFFLOAT(prec)	(unsigned)(1 + (prec>>1))
 
-struct bigcell {	/* for big integers */
-	byte	flag;		/* = fBIGNUM */
+struct bigcell {	/* for big integers or gf2nint's */
+	byte	flag;		/* = fBIGNUM or fGF2NINT */
 	byte	signum;		/* same position as in floatcell */
 	word2	len;		/* same position as in vector */
 	word2	digi0;
@@ -499,8 +542,8 @@ struct bigcell {	/* for big integers */
 
 typedef struct {
 	long	expo;
-	int	sign;
-	int	len;
+	int	    sign;
+	int	    len;
 	word2	*digits;
 } numdata;
 
@@ -548,17 +591,17 @@ struct stream {	   /* I/O stream structure */
 	byte mode;	/* one of INSTREAM,OUTSTREAM,NOSTREAM */
 	int2 pos;	/* current position in line */
 	int4 lineno;	/* current line number */
-	int2 ch;	/* current character */
-	int2 tok;	/* current token */
+	int4 ch;	/* current character */
+	int4 tok;	/* current token */
 	FILE *file;	/* the file associated with stream */
 };
-#define OFFSETmode	1
-#define OFFSETpos	2
+#define OFFSETmode		1
+#define OFFSETpos		2
 #define OFFSETlineno	4
-#define OFFSETch	8
-#define OFFSETtok      10
-#define OFFSETfile     12
-#define SIZEOFSTREAM	4	/* unit is sizeof(truc) */
+#define OFFSETch		8
+#define OFFSETtok      12
+#define OFFSETfile     16
+#define SIZEOFSTREAM	(sizeof(struct stream)/sizeof(truc))
 
 struct stack {
 	byte flag;
@@ -608,12 +651,12 @@ struct fundef {
 	truc	parms;		/* default initializations of formal args */
 	truc	vars;		/* list of initializations of local vars */
 };
-#define OFFSETfargc	2
+#define OFFSETfargc		2
 #define OFFSETvarcount	6
-#define OFFSETbody	8
+#define OFFSETbody		8
 #define OFFSETparms    12
 #define OFFSETvars     16
-#define OFFS4body	2
+#define OFFS4body		2
 #define SIZEOFFUNDEF	5		/* unit is sizeof(truc) */
 
 struct compnode {	/* compound statement */
@@ -653,7 +696,7 @@ struct fornode {
 #define symptr(x)	((struct symbol *)Saddress(x))			  
 #define streamptr(x)	((struct stream *)Taddress(x))			  
 #define stringptr(x)	((struct strcell *)Taddress(x))			  
-#define recordptr(x)	((struct record *)Taddress(x))			  
+#define recordptr(x)	((struct record *)Taddress(x))
 								  
 #define FLAG(x)		*(byte *)&(x)					  
 #define STRING(x)	(char *)(bTaddress(x) + OFFSETstring)		  
@@ -679,12 +722,13 @@ struct fornode {
 #define STREAMPTR(p)	((struct stream *)TAddress(p))
 #define STRCELLPTR(p)	((struct strcell *)TAddress(p))
 #define RECORDPTR(p)	((struct record *)TAddress(p))
+#define VECSTRUCTPTR(p) ((struct vector *)TAddress(p))
 #define STREAMTOKPTR(p)	 ((int2 *)(bTAddress(p) + OFFSETtok))
 #define STREAMMODEPTR(p) ((byte *)(bTAddress(p) + OFFSETmode))
 #define STREAMLINOPTR(p) ((int4 *)(bTAddress(p) + OFFSETlineno))
 #define STRLENPTR(p)	((word2 *)(bTAddress(p) + OFFSETstrlen))
 #define STRINGPTR(p)	((char *)(bTAddress(p) + OFFSETstring))
-#define BYTEPTR(p)	((byte *)(bTAddress(p) + OFFSETstring))
+#define BYTEPTR(p)		((byte *)(bTAddress(p) + OFFSETstring))
 #define VECLENPTR(p)	((word2 *)(bTAddress(p) + OFFSETveclen))
 #define VECTORPTR(p)	((truc *)(bTAddress(p) + OFFSETvector))
 #define PTARGETPTR(p)	((truc *)(bTAddress(p) + OFFSETfield1))
@@ -694,8 +738,8 @@ struct fornode {
 #define FLTEXPOPTR(p)	((int2 *)(bTAddress(p) + OFFSETexpo))
 
 #define ARGCOUNTPTR(p)	((word2 *)(bTAddress(p) + OFFSETargcount))
-#define ARG0PTR(p)	((truc *)(bTAddress(p) + OFFSETarg0))
-#define ARG1PTR(p)	((truc *)(bTAddress(p) + OFFSETarg1))
+#define ARG0PTR(p)		((truc *)(bTAddress(p) + OFFSETarg0))
+#define ARG1PTR(p)		((truc *)(bTAddress(p) + OFFSETarg1))
 #define ARGNPTR(p,n)	((truc *)(bTAddress(p) + OFFSETargn(n)))
 
 #define COMPLENPTR(p)	((word2 *)(bTAddress(p) + OFFSETcomplen))
@@ -703,11 +747,12 @@ struct fornode {
 #define FUNVARCPTR(p)	((word2 *)(bTAddress(p) + OFFSETvarcount))
 #define FUNVARSPTR(p)	((truc *)(bTAddress(p) + OFFSETvars))
 
-#define SYMPTR(p)	((struct symbol *)SAddress(p))
+#define SYMPTR(p)		((struct symbol *)SAddress(p))
 #define SYMFLAGPTR(p)	bSAddress(p)
 #define SYMBINDPTR(p)	((truc *)(bSAddress(p) + OFFSETbind))
+#define SYMWBINDPTR(p)	((wtruc *)(bSAddress(p) + OFFSETbind))
 #define SYMNAMEPTR(p)	(*(char **)(bSAddress(p) + OFFSETname))
-#define SYMCCPTR(p)	((word4 *)(bSAddress(p) + OFFSETcc))
+#define SYMCCPTR(p)		((word4 *)(bSAddress(p) + OFFSETcc))
 #define SYMCC0PTR(p)	((word2 *)(bSAddress(p) + OFFSETcc))
 #define SYMCC1PTR(p)	((word2 *)(bSAddress(p) + OFFSETcc1))
 #define LSYMBOLPTR(p)	(basePtr + *WORD2PTR(p))
@@ -715,7 +760,7 @@ struct fornode {
 #define LSYMFLAGPTR(p)	(byte *)(basePtr + *WORD2PTR(p))
 
 #define STREAMmode(x)	*(byte *)(bTaddress(x) + OFFSETmode)
-#define STREAMch(x)	*(int2 *)(bTaddress(x) + OFFSETch)		 
+#define STREAMch(x)		*(int2 *)(bTaddress(x) + OFFSETch)
 #define STREAMtok(x)	*(int2 *)(bTaddress(x) + OFFSETtok)		 
 #define STREAMpos(x)	*(int2 *)(bTaddress(x) + OFFSETpos)
 #define STREAMfile(x)	*(FILE **)(bTaddress(x) + OFFSETfile)		 
@@ -742,6 +787,8 @@ struct fornode {
 #define WORKpop()	workStkPtr--
 #define WORKnpop(n)	workStkPtr -= (n)
 #define WORKretr()	*workStkPtr--
+#define WORKnpush(n) \
+ 	if(!((workStkPtr += (n)) < evalStkPtr)) reset(err_wrkstk)
 #define WORKspace(n) \
 	(workStkPtr < evalStkPtr-(n)-32 ? workStkPtr += (n) : NULL)
 
@@ -772,7 +819,9 @@ struct fornode {
 /*--------------------- external declarations -----------------------*/
 #ifdef Win32GUI
 #include "ariwin.h"
-
+#endif
+#ifdef GtK
+#include "gnariwin.h"
 #endif
 
 /* errtext.c */
@@ -783,20 +832,22 @@ extern char	*err_funest, *err_funame, *err_2ident,
 		*err_memory, *err_2large, *err_memev, *err_garb,
 		*err_evstk, *err_wrkstk, *err_astk, *err_savstk, *err_pstk,
 		*err_imp, *err_case, *err_rec, *err_intr,
-		*err_rparen, *err_0rparen, *err_0lparen, *err_0rbrack,
+		*err_rparen, *err_0rparen, *err_0lparen, 
+		*err_0brace, *err_0rbrack,
 		*err_brstr, *err_bchar, *err_inadm,
 		*err_stkv, *err_stke, *err_stkbig, *err_nil, *err_vpoint,
 		*err_filv, *err_outf, *err_tout, *err_bout,
 		*err_inpf, *err_tinp, *err_binp,
 		*err_then, *err_end,
 		*err_ovfl, *err_div, *err_2big, *err_float, *err_bool, 
-		*err_int, *err_fix, *err_pfix, *err_p4int, *err_odd,
+		*err_int, *err_intt, *err_fix, *err_pfix, *err_p4int,
+        *err_odd, *err_oddprim,
 		*err_char, *err_chr, *err_2long, *err_iovfl,
 		*err_num, *err_pnum, *err_p0num, *err_intvar,
-		*err_pbase, *err_range, *err_irange, *err_var,
+		*err_pbase, *err_range, *err_irange, *err_var, *err_lval,
 		*err_vsym, *err_vasym, *err_sym, *err_gsym, *err_sym2,
 		*err_buf, *err_str, *err_bystr, *err_vbystr,
-		*err_arr, *err_sarr, *err_vect, *err_field, *err_open,
+		*err_arr, *err_syarr, *err_sarr, *err_vect, *err_field, *err_open,
 		*err_bltin, *err_ubound, *err_ufunc;
 
 /* alloc.c */
@@ -819,20 +870,20 @@ extern size_t	hashtabSize, aribufSize, auxbufSize, scrbufSize;
 
 extern truc	*Symbol;
 extern truc	*Memory[];
-extern trucptr	*Symtab;
+extern trucptr *Symtab;
 extern truc	*WorkStack, *evalStkPtr, *workStkPtr;
 extern truc	*ArgStack, *argStkPtr, *saveStkPtr;
 extern truc	*basePtr;
-extern word2	*AriBuf, *AriScratch, *AuxBuf, *PrimTab;
+extern word2 *AriBuf, *AriScratch, *AuxBuf, *PrimTab;
 
 /* array.c */
 extern void iniarray	_((void));
-extern void iniargv	_((int argc, char *argv[]));
+extern void iniargv		_((int argc, char *argv[]));
 extern int stringsplit	_((char *str, char *trenn, word2 *offsets));
-extern int indrange	_((truc *ptr, long len, long *pn0, long *pn1));
+extern int indrange		_((truc *ptr, long len, long *pn0, long *pn1));
 extern truc arrassign	_((truc *arr, truc obj));
 extern truc subarrassign  _((truc *arr, truc obj));
-extern void sortarr	_((truc *arr, unsigned len, ifuntt cmpfun));
+extern void sortarr		_((truc *arr, unsigned len, ifuntt cmpfun));
 extern int bytestraddr	_((truc *ptr, truc **ppbstr, byte **ppch,
 						unsigned *plen));
 extern truc recfassign	_((truc *rptr, truc field, truc obj));
@@ -851,7 +902,10 @@ extern truc	ofsym;
 extern void iniarith	_((void));
 extern truc  addints	_((truc *ptr, int minflg));
 extern unsigned random2 _((unsigned u));
-extern int cmpnums	_((truc *ptr1, truc *ptr2, int type));
+extern unsigned random4 _((unsigned u));
+extern int cmpnums		_((truc *ptr1, truc *ptr2, int type));
+extern truc scalintvec 	_((truc *ptr1, truc *ptr2));
+extern truc Gvecmod     _((int flg));
 
 extern truc	integsym, int_sym, realsym;
 extern truc	zero, constone, flt0zero;
@@ -862,22 +916,46 @@ extern truc	ariltsym, arigtsym, arilesym, arigesym, arieqsym, arinesym;
 extern long	maxfltex, maxdecex, exprange;
 
 /* aritx.c */
-extern void	iniaritx  _((void));
-extern int	prime16	  _((unsigned u));
-extern int	prime32	  _((word4 u));
+extern void	iniaritx	_((void));
+extern int	prime16	  	_((unsigned u));
+extern int	prime32	  	_((word4 u));
+extern unsigned fact16    _((word4 u));
 extern unsigned trialdiv  _((word2 *x, int n, unsigned u0, unsigned u1));
-extern int	jac	  _((unsigned x, unsigned y));
-extern int	jacobi	  _((int sign, word2 *x, int n, word2 *y, int m,
+extern int	jac	  		_((unsigned x, unsigned y));
+extern int	jacobi	  	_((int sign, word2 *x, int n, word2 *y, int m,
 			     word2 *hilf));
-extern int	rabtest	  _((word2 *x, int n, word2 *aux));
-extern int	modinverse   _((word2 *x, int n, word2 *y, int m, word2 *zz,
+extern int	rabtest	  	_((word2 *x, int n, word2 *aux));
+extern int nextprime32 _((word4 u, word2 *x));
+extern int pemult	_((word2 *x, int n, word2 *ex, int exlen,
+			   word2 *aa, int alen,
+			   word2 *mm, int modlen, word2 *z, word2 *hilf));
+extern int	modinverse	_((word2 *x, int n, word2 *y, int m, word2 *zz,
 				word2 *hilf));
-extern int	modpower     _((word2 *x, int n, word2 *ex, int exlen,
+extern int modinv   _((int x, int mm));
+extern int	modpower	_((word2 *x, int n, word2 *ex, int exlen,
 				word2 *mm, int modlen, word2 *p, word2 *hilf));
+extern unsigned modpow    _((unsigned x, unsigned n, unsigned mm));
 extern truc	modpowsym;
 
 /* arity.c */
-extern void	iniarity  _((void));
+extern void	iniarity  	_((void));
+extern void workmess   _((void));
+extern void tick   	_((int c));
+
+/* aritz.c */
+extern void	iniaritz	_((void));
+extern truc gf2nzero, gf2none, gf2nintsym, gf2n_sym;
+extern truc polmultsym, polNmultsym, polmodsym, polNmodsym,
+        poldivsym, polNdivsym;
+extern truc addgf2ns    _((truc *ptr));
+extern truc multgf2ns   _((truc *ptr));
+extern truc divgf2ns    _((truc *ptr));
+extern truc exptgf2n    _((truc *ptr));
+extern int fpSqrt   _((word2 *pp, int plen, word2 *aa, int alen,
+                    word2 *zz, word2 *hilf));
+extern int fp2Sqrt     _((word2 *pp, int plen, word2 *aa, int alen,
+                word2 *zz, word2 *hilf));
+extern unsigned fp_sqrt    _((unsigned p, unsigned a));
 
 /* aritaux.c */
 extern int FltPrec[];
@@ -885,40 +963,47 @@ extern int MaxFltLevel;
 extern int setfltprec	_((int prec));
 extern int deffltprec	_((void));
 extern int maxfltprec	_((void));
-extern int fltprec	_((int type));
+extern int fltprec		_((int type));
 extern int fltpreccode	_((int prec));
 extern int refnumtrunc	_((int prec, truc *ptr, numdata *nptr));
 extern int getnumtrunc	_((int prec, truc *ptr, numdata *nptr));
 extern int getnumalign	_((int prec, truc *ptr, numdata *nptr));
 extern int alignfloat	_((int prec, numdata *nptr));
-extern int alignfix	_((int prec, numdata *nptr));
+extern int alignfix		_((int prec, numdata *nptr));
 extern void adjustoffs	_((numdata *npt1, numdata *npt2));
 extern int normfloat	_((int prec, numdata *nptr));
 extern int multtrunc	_((int prec, numdata *npt1, numdata *npt2,
 			   word2 *hilf));
-extern int divtrunc	_((int prec, numdata *npt1, numdata *npt2,
+extern int divtrunc		_((int prec, numdata *npt1, numdata *npt2,
 			   word2 *hilf));
-extern int pwrtrunc	_((int prec, unsigned base, unsigned a,
+extern int pwrtrunc		_((int prec, unsigned base, unsigned a,
 			   numdata *nptr, word2 *hilf));
 extern int float2bcd	_((int places, truc *p, numdata *nptr,
 			   word2 *hilf));
-extern int roundbcd	_((int prec, numdata *nptr));
+extern int roundbcd		_((int prec, numdata *nptr));
 extern int flodec2bin	_((int prec, numdata *nptr, word2 *hilf));
 extern void int2numdat	_((int x, numdata *nptr));
 extern void cpynumdat	_((numdata *npt1, numdata *npt2));
 extern int numposneg	_((truc *ptr));
 extern truc wipesign	_((truc *ptr));
-extern long intretr	_((truc *ptr));
-extern int bigref	_((truc *ptr, word2 **xp, int *sp));
+extern truc changesign	_((truc *ptr));
+extern long intretr		_((truc *ptr));
+extern int bigref		_((truc *ptr, word2 **xp, int *sp));
 extern int bigretr	_((truc *ptr, word2 *x, int *sp));
 extern int twocretr	_((truc *ptr, word2 *x));
 extern int and2arr	_((word2 *x, int n, word2 *y, int m));
 extern int or2arr	_((word2 *x, int n, word2 *y, int m));
 extern int xor2arr	_((word2 *x, int n, word2 *y, int m));
+extern int xorbitvec	_((word2 *x, int n, word2 *y, int m));
 extern long bit_length	_((word2 *x, int n));
+extern int chkintnz _((truc sym, truc *ptr));
 extern int chkints	_((truc sym, truc *argptr, int n));
+extern int chkint   _((truc sym, truc *ptr));
+extern int chkintt  _((truc sym, truc *ptr));
 extern int chknums	_((truc sym, truc *argptr, int n));
 extern int chknum	_((truc sym, truc *ptr));
+extern int chkintvec	_((truc sym, truc *vptr));
+extern int chknumvec	_((truc sym, truc *vptr));
 
 /* arito386.asm */
 #ifdef M_3264
@@ -955,6 +1040,7 @@ extern word4 big2long	_((word2 *x, int n));
 extern word4 intsqrt	_((word4 u));
 extern int bitlen	_((unsigned x));
 extern int niblen	_((unsigned x));
+extern int bitcount     _((unsigned u));
 
 /* aritools.c */
 extern int shiftarr	_((word2 *x, int n, int sh));
@@ -962,15 +1048,20 @@ extern int lshiftarr	_((word2 *x, int n, long sh));
 extern int addarr	_((word2 *x, int n, word2 *y, int m));
 extern int subarr	_((word2 *x, int n, word2 *y, int m));
 extern int sub1arr	_((word2 *x, int n, word2 *y, int m));
+extern int addsarr  _((word2 *x, int n, int sign1,
+                    word2 *y, int m, int sing2, int *psign));
 extern int multbig	_((word2 *x, int n, word2 *y, int m, word2 *z,
-			   word2 *hilf));
+                    word2 *hilf));
 extern int divbig	_((word2 *x, int n, word2 *y, int m, word2 *quot,
-			  int *rlenptr, word2 *hilf));
+                    int *rlenptr, word2 *hilf));
 extern int modbig	_((word2 *x, int n, word2 *y, int m, word2 *hilf));
+extern int modnegbig	_((word2 *x, int n, word2 *y, int m, word2 *hilf));
+extern int modmultbig   _((word2 *xx, int xlen, word2 *yy, int ylen,
+                word2 *mm, int mlen, word2 *zz, word2 *hilf));
 extern int multfix	_((int prec, word2 *x, int n, word2 *y, int m,
-			   word2 *z, word2 *hilf));
+                    word2 *z, word2 *hilf));
 extern int divfix	_((int prec, word2 *x, int n, word2 *y, int m,
-			   word2 *z, word2 *hilf));
+                    word2 *z, word2 *hilf));
 extern unsigned shortgcd _((unsigned x, unsigned y));
 extern int biggcd	_((word2 *x, int n, word2 *y, int m, word2 *hilf));
 extern int power	_((word2 *x, int n, unsigned a, word2 *p,
@@ -1019,6 +1110,7 @@ extern truc	tstdout, tstdin, tstderr;
 
 /* control.c: */
 extern void inicont	_((void));
+extern int is_lval  _((truc *ptr));
 extern int Lvaladdr	_((truc *ptr, trucptr *pvptr));
 extern truc Lvalassign	_((truc *ptr, truc obj));
 extern truc Swhile	_((void));
@@ -1049,6 +1141,11 @@ extern truc	boolsym, truesym, falsesym, true, false, nil;
 extern truc	usersym, arisym, symbsym;
 
 /* mainloop.c */
+#ifdef DTRACE
+extern FILE *DTraceF;
+extern int DTraceWrite(char *mess);
+extern char DTraceZeile[80];
+#endif
 extern truc	helpsym;
 extern truc	apathsym;
 extern truc	*res1Ptr, *res2Ptr, *res3Ptr;
@@ -1111,6 +1208,7 @@ extern void iniprint	_((int cols));
 extern int logout	_((int ch));
 extern void strlogout	_((char *str));
 extern void closelog	_((void));
+extern void flushlog	_((void));
 extern int setprnprec	_((int prec));
 extern void tprint	_((truc strom, truc obj));
 extern int strcopy	_((char *tostr, char *fromstr));
@@ -1120,10 +1218,11 @@ extern void fprintline	_((truc strom, char *str));
 extern void fnewline	_((truc strom));
 extern void ffreshline	_((truc strom));
 extern void flinepos0	_((truc strom));
-extern int s1form	_((char *buf, char *fmt, word4 dat));
-extern int s2form	_((char *buf, char *fmt, word4 dat1, word4 dat2));
+extern int s1form	_((char *buf, char *fmt, wtruc dat));
+extern int s2form	_((char *buf, char *fmt, wtruc dat1, wtruc dat2));
 
 extern truc	writesym, writlnsym, formatsym;
+extern truc transcsym;
 extern char	OutBuf[];
 extern int	Log_on;
 
@@ -1136,11 +1235,11 @@ extern truc mksym	_((char *name, int *sflgptr));
 extern truc scratch	_((char *name));
 extern truc newselfsym	_((char *name, int flg));
 extern truc newreflsym	_((char *name, int flg));
-extern truc newintsym	_((char *name, int flg, truc bind));
+extern truc newintsym	_((char *name, int flg, wtruc bind));
 extern int  tokenvalue	_((truc op));
 extern truc newsym	_((char *name, int flg, truc bind));
-extern truc newsymsig	_((char *name, int flg, truc bind, int sig));
-extern truc new0symsig	_((char *name, int flg, truc bind, int sig));
+extern truc newsymsig	_((char *name, int flg, wtruc bind, int sig));
+extern truc new0symsig	_((char *name, int flg, wtruc bind, int sig));
 extern truc mkcopy	_((truc *x));
 extern truc mkarrcopy	_((truc *x));
 extern truc mkinum	_((long n));
@@ -1149,7 +1248,8 @@ extern truc mklocsym	_((int flg, unsigned u));
 extern truc mkfixnum	_((unsigned n));
 extern truc mksfixnum	_((int n));
 extern truc mkint	_((int sign, word2 *arr, int len));
-extern truc mk0big	_((int sign, word2 *arr, int len));
+extern truc mkgf2n  _((word2 *arr, int len));
+extern truc mk0gf2n _((word2 *arr, int len));
 extern truc mkfloat	_((int prec, numdata *nptr));
 extern truc fltzero	_((int prec));
 extern truc mk0float	_((numdata *nptr));
@@ -1182,16 +1282,18 @@ extern void dumpinput	_((void));
 extern char *treadline	_((void));
 extern void historyout	_((int flg));
 
-extern truc historsym, savinsym;
+extern truc historsym, savinsym, bufovflsym;
 
 /* sysdep.c */
 extern void stacklimit	_((void));
 extern long stkcheck	_((void));
-extern long timer	_((void));
-extern int sysrand	_((void));
+extern long timer	    _((void));
+extern long datetime	_((int tim[6]));
+extern int sysrand	    _((void));
 extern void prologue	_((void));
-extern int epilogue	_((void));
-
+extern int epilogue	    _((void));
+extern char *getworkdir _((void));
+extern int setworkdir   _((char *pfad));
 #ifdef ATARIST
 extern int	VDI_handle;
 #endif
@@ -1202,7 +1304,7 @@ extern int chknargs	_((truc fun, int n));
 
 extern int	s_dum, s_0, s_01, s_02, s_0u, s_1, s_1u, s_12, s_bV,
 		s_rr, s_vr, s_ii, s_iI, s_bs, s_nv, s_rrr, s_iii,
-		s_12ii, s_12rn, s_13, s_2, s_23, s_3, s_0uii,
+		s_12ii, s_12rn, s_13, s_14, s_2, s_23, s_3, s_0uii,
 		s_iiii, s_4, s_Viiii, s_iiiII;
 
 #define NARGS_FALSE	0
