@@ -2,7 +2,7 @@
 /* file control.c
 
 ARIBAS interpreter for Arithmetic
-Copyright (C) 1996 O.Forster
+Copyright (C) 1996-2002 O.Forster
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ Address of the author
 	Theresienstr. 39
 	D-80333 Muenchen, Germany
 
-Email	forster@rz.mathematik.uni-muenchen.de
+Email	forster@mathematik.uni-muenchen.de
 */
 /****************************************************************/
 
@@ -34,20 +34,25 @@ Email	forster@rz.mathematik.uni-muenchen.de
 ** function definition, logical and control functions
 **
 ** date of last change
-** 95-02-22: lpbrksym
-** 95-03-15: const
-** 95-03-20: changed make_unbound
-** 95-03-31: pointer
-** 97-04-22: type symbol, reorg (newintsym), changed Sfor
-** 97-08-18: removed bug (discovered by M.Zimmer, Leipzig) in Sfor
-** 98-01-17: small change in Sfor (regarding toolong)
-** 98-10-07  continue statement
-** 99-06-21  make_unbound(user)
+** 1995-02-22: lpbrksym
+** 1995-03-15: const
+** 1995-03-20: changed make_unbound
+** 1995-03-31: pointer
+** 1997-04-22: type symbol, reorg (newintsym), changed Sfor
+** 1997-08-18: removed bug (discovered by M.Zimmer, Leipzig) in Sfor
+** 1998-01-17: small change in Sfor (regarding toolong)
+** 1998-10-07: continue statement
+** 1999-06-21: make_unbound(user)
+** 2002-03-27: small change in Lvalassign, new function is_lval()
+** 2002-04-04: simultaneous assignment (x1,...,xn) := (val1,...,valn)
+** 2002-04-27: gmtime
+** 2003-02-28: case fGF2NINT in nulltest()
 */
 
 #include "common.h"
 
 PUBLIC void inicont	_((void));
+PUBLIC int is_lval  _((truc *ptr));
 PUBLIC int Lvaladdr	_((truc *ptr, trucptr *pvptr));
 PUBLIC truc Lvalassign	_((truc *ptr, truc obj));
 PUBLIC truc Swhile	_((void));
@@ -83,7 +88,7 @@ PUBLIC truc beginsym, endsym;
 PUBLIC truc *brkbindPtr, *brkmodePtr;
 /*--------------------------------------------------*/
 PRIVATE truc symbolssym;
-PRIVATE truc timersym;
+PRIVATE truc timersym, gmtimsym;
 PRIVATE truc mkunbdsym;
 PRIVATE truc andsym, orsym;
 
@@ -105,7 +110,8 @@ PRIVATE int increment	_((word2 *x, int n, int *signptr,
 PRIVATE truc Freturn	_((void));
 PRIVATE truc Slpbreak	_((void));
 PRIVATE truc Slpcont	_((void));
-PRIVATE truc Stimer	_((void));
+PRIVATE truc Stimer	    _((void));
+PRIVATE truc Fgmtime    _((int argn));
 PRIVATE truc Smkunbound	  _((void));
 PRIVATE truc Fsymbols	_((void));
 PRIVATE int symbcmp	_((truc *ptr1, truc *ptr2));
@@ -133,13 +139,13 @@ PUBLIC void inicont()
 	errsym	   = newselfsym("error",sINTERNAL);
 
 	exitsym	   = newsym("exit",   sPARSAUX, nullsym);
-	temp	   = newintsym("exit",sSBINARY, (truc)Sexit);
+	temp	   = newintsym("exit",sSBINARY, (wtruc)Sexit);
 	exitfun	   = mk0fun(temp);
 	lpbrksym   = newsym("break",  sPARSAUX, nullsym);
-	temp	   = newintsym("break",sSBINARY, (truc)Slpbreak);
+	temp	   = newintsym("break",sSBINARY, (wtruc)Slpbreak);
 	lpbrkfun   = mk0fun(temp);
 	lpcontsym  = newsym("continue",  sPARSAUX, nullsym);
-	temp	   = newintsym("continue",sSBINARY, (truc)Slpcont);
+	temp	   = newintsym("continue",sSBINARY, (wtruc)Slpcont);
 	lpcontfun  = mk0fun(temp);
 
 	breaksym   = newsym("$break", sINTERNVAR, voidsym);
@@ -147,17 +153,17 @@ PUBLIC void inicont()
 	brkmodePtr = (truc *)SYMCCPTR(&breaksym);
 	*brkmodePtr = breaksym;
 
-	equalsym   = newintsym("=",  sFBINARY, (truc)Fequal);
-	nequalsym  = newintsym("/=", sFBINARY, (truc)Fnequal);
+	equalsym   = newintsym("=",  sFBINARY, (wtruc)Fequal);
+	nequalsym  = newintsym("/=", sFBINARY, (wtruc)Fnequal);
 
-	assignsym  = newintsym(":= ", sSBINARY, (truc)Sassign);
+	assignsym  = newintsym(":= ", sSBINARY, (wtruc)Sassign);
 
 	funcsym	   = newsym("function", sPARSAUX, nullsym);
 	procsym	   = newsym("procedure",sPARSAUX, nullsym);
 	extrnsym   = newsym("external", sDELIM,	  nullsym);
 	varsym	   = newsym("var",	sPARSAUX, nullsym);
-	var_sym	   = newintsym("_var",	sSBINARY,(truc)Svarparm);
-	inivarsym  = newintsym("var",	sSBINARY,(truc)Sinivars);
+	var_sym	   = newintsym("_var",	sSBINARY,(wtruc)Svarparm);
+	inivarsym  = newintsym("var",	sSBINARY,(wtruc)Sinivars);
 	constsym   = newsym("const",	sPARSAUX, nullsym);
 	const_sym  = newsym("$const", sINTERNVAR, voidsym);
 	constbindPtr = SYMBINDPTR(&const_sym);
@@ -175,22 +181,24 @@ PUBLIC void inicont()
 	beginsym   = newsym("begin", sDELIM,   nullsym);
 	endsym	   = newsym("end",   sDELIM,   nullsym);
 
-	not_sym	   = newintsym("not",sFBINARY, (truc)Fnot);
+	not_sym	   = newintsym("not",sFBINARY, (wtruc)Fnot);
 	notsym	   = newsym("not",   sPARSAUX, not_sym);
 
-	ret_sym	   = newintsym("return",sFBINARY, (truc)Freturn);
+	ret_sym	   = newintsym("return",sFBINARY, (wtruc)Freturn);
 	retsym	   = newsym("return",	sPARSAUX, ret_sym);
 
-	and_sym	   = newintsym("and",sSBINARY, (truc)Sand);
-	andsym	   = newsymsig("and",	sINFIX, and_sym, ANDTOK);
+	and_sym	   = newintsym("and",sSBINARY, (wtruc)Sand);
+	andsym	   = newsym("and",	sINFIX, and_sym);
+    SYMcc1(andsym) = ANDTOK;
 
-	or_sym	   = newintsym("or", sSBINARY, (truc)Sor);
-	orsym	   = newsymsig("or",	sINFIX, or_sym, ORTOK);
+	or_sym	   = newintsym("or", sSBINARY, (wtruc)Sor);
+	orsym	   = newsym("or",	sINFIX, or_sym); 
+    SYMcc1(orsym) = ORTOK;
 
 	truesym	   = newsym("true",  sSCONSTANT, true);
 	falsesym   = newsym("false", sSCONSTANT, false);
 
-	nil	   = newreflsym("nil",	  sSCONSTANT);
+	nil	       = newreflsym("nil",	  sSCONSTANT);
 	arisym	   = newreflsym("aribas", sSYSSYMBOL);
 	usersym	   = newreflsym("user",	  sSYSSYMBOL);
 #ifdef DEVEL
@@ -198,10 +206,11 @@ PUBLIC void inicont()
 #else
 	symbsym = nullsym;
 #endif
-	timersym   = newsymsig("timer", sSBINARY, (truc)Stimer, s_0);
+	timersym   = newsymsig("timer", sSBINARY, (wtruc)Stimer, s_0);
+    gmtimsym   = newsymsig("gmtime",sFBINARY, (wtruc)Fgmtime,s_01);
 	mkunbdsym  = newsymsig("make_unbound", sSBINARY,
-				(truc)Smkunbound, s_bV);
-	symbolssym = newsymsig("symbols", sFBINARY, (truc)Fsymbols, s_1);
+				(wtruc)Smkunbound, s_bV);
+	symbolssym = newsymsig("symbols", sFBINARY, (wtruc)Fsymbols, s_1);
 }
 /*----------------------------------------------------------------------*/
 PRIVATE truc Fequal()
@@ -245,7 +254,9 @@ truc *ptr1, *ptr2;
 		}
 		return(0);
 	}
-	else switch(flg) {
+	else switch(flg) {  /* here flg == flg2 */
+    case fGF2NINT:
+		return(cmpnums(ptr1,ptr2,flg) ? 0 : 1);
 	case fSTRING:
 	case fBYTESTRING:
 		n = *STRLENPTR(ptr1);
@@ -352,6 +363,8 @@ truc obj;
 		return(v.pp.ww ? 1 : 0);
 	case fBIGNUM:		/* bignum is not zero */
 		return(1);
+    case fGF2NINT:
+        return (obj == gf2nzero ? 0 : 1);
 	default:
 		return(aERROR);
 	}
@@ -375,7 +388,7 @@ PRIVATE truc Sinivars()
 		ptr = TAddress(workStkPtr) + i;
 		sptr = SYMPTR(ptr);
 		*FLAGPTR(sptr) = sVARIABLE;
-		sptr->bind = obj;
+		sptr->bind.t = obj;
 	}
 	ARGpop();
 	WORKpop();
@@ -486,6 +499,8 @@ PRIVATE truc Sassign()
 ** ptr[0] = rec_sym, ptr[1] = Record, ptr[2] = field
 ** vPOINTREF:
 ** ptr[0] = derefsym, ptr[1] = Pointer
+** vVECTOR:
+** ptr[0] = vectorsym, ptr[1] = len, ptr[2] = ele0, ptr[3] = ele1, ..
 ** Die Argumente in ptr[1] bzw. ptr[2] sind jeweils unausgewertet.
 */
 PUBLIC int Lvaladdr(ptr,pvptr)
@@ -512,8 +527,8 @@ trucptr *pvptr;
 		}
 		/* else fall through */
 	}
-	if(flag == fBUILTIN2 || flag == fSPECIAL2 || flag == fSPECIAL1) {
-		/* array access or record access or pointer reference */
+	if(flag >= fSPECIAL1 && flag <= fBUILTINn) {
+		/* array access or record access or pointer reference or vector*/
 		*pvptr = ptr = TAddress(ptr);
 		if(*ptr == arr_sym) {
 			return(vARRELE);
@@ -527,6 +542,9 @@ trucptr *pvptr;
 		else if(*ptr == derefsym) {
 			return(vPOINTREF);
 		}
+		else if(*ptr == vectorsym) {
+			return(vVECTOR);
+		}
 	}
 	/* else aERROR */
 	*pvptr = NULL;
@@ -534,6 +552,66 @@ trucptr *pvptr;
 }
 /*-----------------------------------------------------------*/
 /*
+** stripped down version of Lvaladdr
+*/
+PUBLIC int is_lval(ptr)
+truc *ptr;
+{
+	struct symbol *sptr;
+	int flag;
+
+	flag = *FLAGPTR(ptr);
+	if(flag == fLSYMBOL) {
+		return(vBOUND);
+	}
+	else if(flag == fSYMBOL) {
+        goto symbol;
+	}
+	else if(flag == fRSYMBOL) {
+		ptr = LSYMBOLPTR(ptr);
+		if((flag = *FLAGPTR(ptr)) == fSYMBOL)
+			goto symbol;
+		else if(flag == fLRSYMBOL) {
+			return(vBOUND);
+		}
+		/* else fall through */
+	}
+	if(flag == fBUILTIN2 || flag == fSPECIAL2 || flag == fSPECIAL1) {
+		/* array access or record access or pointer reference */
+		ptr = TAddress(ptr);
+		if(*ptr == arr_sym) {
+			return(vARRELE);
+		}
+		else if(*ptr == subarrsym) {
+			return(vSUBARRAY);
+		}
+		else if(*ptr == rec_sym) {
+			return(vRECFIELD);
+		}
+		else if(*ptr == derefsym) {
+			return(vPOINTREF);
+		}
+		/* else fall through */
+	}
+	/* else aERROR */
+	return(aERROR);
+  symbol:
+	sptr = SYMPTR(ptr);
+	switch(*FLAGPTR(sptr)) {
+    case sUNBOUND:
+		return(vUNBOUND);
+	case sVARIABLE:
+		return(vBOUND);
+	case sCONSTANT:
+	case sSCONSTANT:
+		return(vCONST);
+	default:
+		break;
+	}
+	return(aERROR);
+}
+/*-----------------------------------------------------------*/
+/*
 ** Legt in *pvptr die Adresse, in der der Wert des Symbols
 ** gespeichert ist, falls es sich um eine Variable oder ein
 ** ungebundenes Symbol handelt
@@ -551,18 +629,18 @@ trucptr *pvptr;
 
 	sptr = SYMPTR(ptr);
 	switch(*FLAGPTR(sptr)) {
-	    case sUNBOUND:
+    case sUNBOUND:
 		*FLAGPTR(sptr) = sVARIABLE;
-		*pvptr = &(sptr->bind);
+		*pvptr = &(sptr->bind.t);
 		return(vUNBOUND);
-	    case sVARIABLE:
-		*pvptr = &(sptr->bind);
+	case sVARIABLE:
+		*pvptr = &(sptr->bind.t);
 		return(vBOUND);
-	    case sCONSTANT:
-	    case sSCONSTANT:
+	case sCONSTANT:
+	case sSCONSTANT:
 		*pvptr = NULL;
 		return(vCONST);
-	    default:
+	default:
 		*pvptr = NULL;
 		return(aERROR);
 	}
@@ -573,8 +651,9 @@ truc *ptr;
 truc obj;
 {
 	truc *vptr;
-	truc *ptr1;
-	int flg, ret;
+	truc *ptr1, *work0ptr;
+    truc ele;
+	int flg, ret, len, k;
 
 	if(obj == nil)
 		return(Pdispose(ptr));
@@ -585,17 +664,17 @@ truc obj;
 		case fRECORD:
 			return(fullrecassign(vptr,obj));
 		case fPOINTER:
-			;
-		}
+        default:
 /******* type check unvollstaendig **********/
-		return(*vptr = obj);
+    		return(*vptr = obj);
+		}
 	}
 	/* else */
 	WORKpush(obj);
 	switch(ret) {
-	    case vARRELE:
-	    case vSUBARRAY:
-		ARGpush(vptr[1]);
+    case vARRELE:
+	case vSUBARRAY:
+        ARGpush(vptr[1]);
 		ARGpush(vptr[2]);
 		argStkPtr[-1] = eval(argStkPtr-1);
 		argStkPtr[0] = eval(argStkPtr);
@@ -605,14 +684,14 @@ truc obj;
 			obj = subarrassign(argStkPtr-1,*workStkPtr);
 		ARGnpop(2);
 		break;
-	    case vRECFIELD:
+	case vRECFIELD:
 		ARGpush(vptr[1]);
 		*argStkPtr = eval(argStkPtr);
 		obj = recfassign(argStkPtr,vptr[2],*workStkPtr);
 			/* vptr[2] = field */
 		ARGpop();
 		break;
-	    case vPOINTREF:
+	case vPOINTREF:
 		ARGpush(vptr[1]);
 		*argStkPtr = eval(argStkPtr);
 		flg = *FLAGPTR(argStkPtr);
@@ -627,14 +706,35 @@ truc obj;
 			}
 		}
 		else {
-			goto errexit;
+			obj = brkerr();
 		}
 		ARGpop();
 		break;
-	    case vCONST:
+    case vVECTOR:
+        if(*FLAGPTR(workStkPtr) != fVECTOR) {
+            error(assignsym,err_vect,obj);
+            goto errexit;
+        }
+        len = *WORD2PTR(vptr+1);
+        if(len != *VECLENPTR(workStkPtr)) {
+            error(assignsym,"vectors must have same length",mkfixnum(len));
+            goto errexit;
+        }
+        work0ptr = workStkPtr;
+        for(k=0; k<len; k++)
+            WORKpush(vptr[k+2]);
+        for(k=0; k<len; k++) {
+			ele = *(VECTORPTR(work0ptr)+k);
+            ele = Lvalassign(work0ptr+k+1,ele);
+            if(ele == breaksym)
+                goto errexit;
+        }
+        workStkPtr = work0ptr;
+		break;
+	case vCONST:
 		error(assignsym,err_sym2,*ptr);
 		goto errexit;
-	    default:
+	default:
 		error(assignsym,err_vsym,*ptr);
   errexit:
 		obj = brkerr();
@@ -703,7 +803,7 @@ PUBLIC truc Sfor()
 	if((flg = *FLAGPTR(ptr)) == fSYMBOL) {
 		sptr = SYMPTR(ptr);
 		*FLAGPTR(sptr) = sVARIABLE;
-		runvar = &(sptr->bind);
+		runvar = &(sptr->bind.t);
 	}
 	else if(flg == fLSYMBOL) {
 		runvar = LSYMBOLPTR(ptr);
@@ -885,6 +985,34 @@ PRIVATE truc Stimer()
 	return mkinum(timer());
 }
 /*----------------------------------------------------------*/
+PRIVATE truc Fgmtime(argn)
+int argn;
+{
+    int tim[6];
+    long secs;
+    char *str;
+    word4 x,y;
+
+    secs = datetime(tim);
+
+    if(argn == 1 && *argStkPtr == zero) {
+        return mkinum(secs);
+    }
+    /* else */
+    x = tim[0] + 1900;
+    y = tim[1] + 1;
+    str = OutBuf;
+    str += s2form(str,"~04D:~02D:", (wtruc)x,(wtruc)y);
+    x = tim[2];
+    y = tim[3];
+    str += s2form(str,"~02D:~02D:",(wtruc)x,(wtruc)y);
+    x = tim[4];
+    y = tim[5];
+    s2form(str,"~02D:~02D",(wtruc)x,(wtruc)y);
+
+    return mkstr(OutBuf);
+}
+/*----------------------------------------------------------*/
 PRIVATE truc Smkunbound()
 {
 	truc *ptr;
@@ -942,7 +1070,7 @@ truc *ptr;
 	struct symbol *sptr;
 
 	sptr = SYMPTR(ptr);
-	sptr->bind = zero;
+	sptr->bind.t = zero;
 	*FLAGPTR(sptr) = sUNBOUND;
 	return(true);
 }
@@ -984,6 +1112,6 @@ PRIVATE truc Fsymbols()
 PRIVATE int symbcmp(ptr1,ptr2)
 truc *ptr1, *ptr2;
 {
-	return(strcmp(SYMNAMEPTR(ptr1),SYMNAMEPTR(ptr2)));
+	return(strcmp(SYMNAMEPTR(ptr2),SYMNAMEPTR(ptr1)));
 }
 /*********************************************************************/
