@@ -2,7 +2,7 @@
 /* file mainloop.c
 
 ARIBAS interpreter for Arithmetic
-Copyright (C) 1996 O.Forster
+Copyright (C) 1996-2002 O.Forster
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ Address of the author
 	Theresienstr. 39
 	D-80333 Muenchen, Germany
 
-Email	forster@rz.mathematik.uni-muenchen.de
+Email	forster@mathematik.uni-muenchen.de
 WWW	http://www.mathematik.uni-muenchen.de/~forster
 
 The latest version of ARIBAS can be obtained by anonymous ftp from
@@ -74,13 +74,18 @@ static char architec[] = "???";
 #endif
 
 #ifdef MYFUN
-static char Version[] = "1.20(+), Jan. 2001";
+static char Version[] = "1.44(+), November 2003";
 #else
-static char Version[] = "1.20, Jan. 2001";
+static char Version[] = "1.44, November 2003";
 #endif
-static int version_no = 120;
-static char Email[] = "forster@rz.mathematik.uni-muenchen.de";
+static int version_no = 144;
+static char Email[] = "forster@mathematik.uni-muenchen.de";
 
+#ifdef DTRACE
+PUBLIC FILE *DTraceF;
+PUBLIC int DTraceWrite(char *mess);
+PUBLIC char DTraceZeile[80];
+#endif
 /*--------------------------------------------------------------*/
 
 /********* prototypes of exported functions ************/
@@ -180,10 +185,20 @@ char *argv[];
 	int ret;
 
 	ret = main0(argc,argv);
+#ifdef DTRACE
+        DTraceF = fopen("DTraceF.txt","w");
+        if(DTraceF == NULL) {
+                ret = EXITREQ;
+        }
+#endif
 	if(ret != EXITREQ)
 		ret = mainloop();
 	else
 		ret = 0;
+#ifdef DTRACE
+        if(DTraceF)
+                fclose(DTraceF);
+#endif
 	closelog();
 	dealloc();
 	epilogue();
@@ -227,6 +242,14 @@ char *argv[];
 	return(ret);
 }
 /*------------------------------------------------------------------*/
+#ifdef DTRACE
+PUBLIC DTraceWrite(mess)
+char *mess;
+{
+     fprintf(DTraceF,mess);
+}
+#endif
+/*------------------------------------------------------------------*/
 PRIVATE void iniopt(popt,argv0)
 struct options *popt;
 char *argv0;
@@ -497,7 +520,7 @@ struct options *popt;
 	int sflg;
 	char *str;
 
-	helpsym	   = newsymsig("help",sSBINARY, (truc)Shelp, s_01);
+	helpsym	   = newsymsig("help",sSBINARY, (wtruc)Shelp, s_01);
 
 	if(popt->helppath != NULL)
 		str = popt->helppath;
@@ -518,16 +541,16 @@ struct options *popt;
 	res3sym	   = newsym("___",sSYSTEMVAR, zero);
 	res3Ptr	   = SYMBINDPTR(&res3sym);
 
-	haltsym	   = newsymsig("halt", sFBINARY, (truc)Fhalt, s_01);
-	verssym	   = newsymsig("version", sFBINARY, (truc)Fversion, s_01);
+	haltsym	   = newsymsig("halt", sFBINARY, (wtruc)Fhalt, s_01);
+	verssym	   = newsymsig("version", sFBINARY, (wtruc)Fversion, s_01);
 
 #ifdef DOSorUNiX
-	systemsym  = newsymsig("system",sFBINARY,(truc)Fsystem,s_1);
-	getenvsym  = newsymsig("getenv",sFBINARY,(truc)Fgetenv,s_1);
+	systemsym  = newsymsig("system",sFBINARY,(wtruc)Fsystem,s_1);
+	getenvsym  = newsymsig("getenv",sFBINARY,(wtruc)Fgetenv,s_1);
 #endif
 
 #ifdef	TT
-	ttsym = newsymsig("tt",sFBINARY,(truc)Ftt,s_0);
+	ttsym = newsymsig("tt",sFBINARY,(wtruc)Ftt,s_0);
 #endif
 }
 /*------------------------------------------------------------------*/
@@ -551,9 +574,9 @@ NULL
 PRIVATE void title()
 {
 	s2form(OutBuf,"~%ARIBAS Interpreter for Arithmetic, V~A (~A)",
-		(word4)Version,(word4)architec);
+		(wtruc)Version,(wtruc)architec);
 	fprintline(tstdout,OutBuf);
-	s1form(OutBuf,"Copyright (C) 1996-2001 O.Forster <~A>",(word4)Email);
+	s1form(OutBuf,"Copyright (C) 1996-2003 O.Forster <~A>",(wtruc)Email);
 	fprintline(tstdout,OutBuf);
 	displaypage(gpltxt);
 	fnewline(tstdout);
@@ -572,12 +595,12 @@ int verb;
 	ret = findarifile(fil,name);
 	if(verb) {
 		fnewline(tstdout);
-		s1form(OutBuf,"(** loading ~A **)",(word4)name);
+		s1form(OutBuf,"(** loading ~A **)",(wtruc)name);
 		fprintline(tstdout,OutBuf);
 	}
 	ret = loadaux(name,verb,NULL);
 	if(ret == aERROR) {
-		s1form(OutBuf,"error while loading file ~A",(word4)name);
+		s1form(OutBuf,"error while loading file ~A",(wtruc)name);
 		fprintline(tstderr,OutBuf);
 	}
 	return(ret);
@@ -639,7 +662,7 @@ int argn;
 {
 	if(argn == 0 || *argStkPtr != zero) {
 		s2form(OutBuf,"ARIBAS Version ~A (~A)",
-			(word4)Version,(word4)architec);
+			(wtruc)Version,(wtruc)architec);
 		fprintline(tstdout,OutBuf);
 	}
 	return(mkfixnum(version_no));
@@ -934,7 +957,7 @@ char *topic;
 		}
 	}
 	else {
-		s1form(OutBuf,"no help available for ~A",(word4)topic);
+		s1form(OutBuf,"no help available for ~A",(wtruc)topic);
 		fprintline(tstdout,OutBuf);
 	}
 	fclose(hfile);
