@@ -25,7 +25,7 @@ Address of the author
 	Theresienstr. 39
 	D-80333 Muenchen, Germany
 
-Email	forster@rz.mathematik.uni-muenchen.de
+Email	forster@mathematik.uni-muenchen.de
 */
 /****************************************************************/
 
@@ -34,12 +34,16 @@ Email	forster@rz.mathematik.uni-muenchen.de
 ** parser
 **
 ** date of last change
-** 95-02-22	lpbrksym
-** 95-03-11	fixed problems with tread
-** 95-03-20	const, type
-** 95-03-29	record, pointer to
-** 97-04-13	type symbol, reorg (newintsym)
-** 98-10-07	continue statement
+** 1995-02-22	lpbrksym
+** 1995-03-11	fixed problems with tread
+** 1995-03-20	const, type
+** 1995-03-29	record, pointer to
+** 1997-04-13	type symbol, reorg (newintsym)
+** 1998-10-07	continue statement
+** 2001-03-11	improved some error messages
+** 2002-02-16   uminsym in negate
+** 2002-02-24   Z1TOK
+** 2003-02-27   enddotsym
 */
 
 #include "common.h"
@@ -100,14 +104,26 @@ PRIVATE truc readif	_((truc *strom));
 PRIVATE truc negate	_((truc obj));
 PRIVATE void recoverr	_((truc *strom));
 PRIVATE int pstat	_((int mode, unsigned flag));
+PRIVATE truc tmptoksym	_((int tok));
 
+PRIVATE truc scansym, enddotsym;
 
 PRIVATE unsigned locsymbs = 0;
 
+#ifdef PARSLOG
+char *plogfile = "pars.log";
+FILE *Plog = NULL;
+char pmess[80];
+#endif
 /*--------------------------------------------------------------------*/
 PUBLIC void iniparse()
 {
-	parserrsym   = newselfsym("syntax error",sINTERNAL);
+	parserrsym = newselfsym("syntax error",sINTERNAL);
+	scansym    = newselfsym("scanning",sINTERNAL);
+    enddotsym  = newselfsym(".",sINTERNAL);
+#ifdef PARSLOG
+    Plog = fopen(plogfile,"w");
+#endif
 }
 /*--------------------------------------------------------------------*/
 /*
@@ -123,7 +139,7 @@ PUBLIC void iniparse()
 #define EXTERNDECL	0x04
 #define POINTRECURS	0x08
 #define PLAINSYM	0x10
-#define ALLFLAG	      0xFFFF
+#define ALLFLAG	    0xFFFF
 
 PRIVATE int pstat(mode,flag)
 int mode;
@@ -256,11 +272,11 @@ int mode;	/* mode = TERMINALINP or FILEINPUT */
 		    }
 		    tok = skipeoltok(strom);
 		    if(tok == SEMICOLTOK) {
-			tok = nexttok(strom,0);
+				tok = nexttok(strom,0);
 		    }
 		    else if(tok == DOTTOK || tok == QUESTIONTOK) {
-			tok = nexttok(strom,0);
-			break;
+				tok = nexttok(strom,0);
+				break;
 		    }
 		}
 		else {		/* FILEINPUT */
@@ -341,7 +357,7 @@ int prec;
 		else if(prec1 == prec) {
 			if(prec == RELPREC) {
 /* Fehler-Behandlung noch mangelhaft */
-				rerror(parserrsym,"",Curop);
+				rerror(parserrsym,"RELPREC",Curop);
 				obj = parserrsym;
 				break;
 			}
@@ -367,31 +383,53 @@ int prec;
 PRIVATE truc optbnode(op)
 truc op;
 {
-	truc obj;
+	truc obj, sym, sym1;
 	truc *ptr;
+    int doopt;
 
 	if(op == modsym) {
 	/* Erkennt einen Ausdruck der Form Base ** Expo mod Modul
 	** und stellt einen Knoten modpower(Base,Expo,Modul) her
+    ** Analog fuer pol_mult(P1,P2) mod N
 	*/
 	    if(*FLAGPTR(argStkPtr-1) == fBUILTIN2) {
-		ptr = TAddress(argStkPtr-1);
-		if(*ptr == powersym) {
-			obj = *argStkPtr;	/* Modul */
-			argStkPtr[-1] = ptr[1]; /* Base */
-			*argStkPtr = ptr[2];	/* Expo */
-			PARSpush(obj);
-			obj = mkfunode(modpowsym,3);
-			PARSpop();
-			return(obj);
-		}
+            doopt = 0;
+			ptr = TAddress(argStkPtr-1);
+            sym = *ptr;
+            if(sym == powersym) {
+                doopt = 1;
+                sym1 = modpowsym;
+            }
+#ifdef POLYARITH
+            else if(sym == polmultsym) {
+                doopt = 1;
+                sym1 = polNmultsym;
+            }
+            else if(sym == polmodsym) {
+                doopt = 1;
+                sym1 = polNmodsym;
+            }
+            else if(sym == poldivsym) {
+                doopt = 1;
+                sym1 = polNdivsym;
+            }
+#endif /* POLYARITH */
+            if(doopt) {
+				obj = *argStkPtr;	    /* Modul */
+				argStkPtr[-1] = ptr[1]; /* arg1 */
+				*argStkPtr = ptr[2];	/* arg2 */
+				PARSpush(obj);
+				obj = mkfunode(sym1,3);
+				PARSpop();
+				return(obj);
+			}
 	    }
 	}
 	/* Binaere Ausdruecke mit konstanten Operanden
 	** werden ausgewertet
 	*/
 	if(op != powersym && isCONSTANT(argStkPtr-1)
-			  && isCONSTANT(argStkPtr)) {
+                      && isCONSTANT(argStkPtr)) {
 		*argStkPtr = mkbnode(op);
 		return(eval(argStkPtr));
 	}
@@ -511,6 +549,9 @@ int *cflgptr;
 	case INUMTOK:
 		obj = mkint(Curnum.sign,Curnum.digits,Curnum.len);
 		break;
+    case GF2NTOK:
+		obj = mkgf2n(Curnum.digits,Curnum.len);
+		break;
 	case FLOATTOK:
 		obj = mkfloat(fltreadprec(),&Curnum);
 		break;
@@ -560,18 +601,22 @@ int *cflgptr;
 		*cflgptr = 1;
 		break;
 	case DOTTOK:
-/*****vorlaeufig*******/
-		obj = voidsym;
+		obj = enddotsym;
 		*cflgptr = 1;
 		weiter = 0;
 		break;
+	case Z1TOK:
+        *cflgptr = 1;
+        error(scansym,"input too long",bufovflsym);
+		obj = brkerr();
+        break;
 	case EOFTOK:
 		obj = eofsym;
 		weiter = 0;
 		break;
 	default:
-		rerror(parserrsym,"tok",mkfixnum(tok));
-/* genauere Fehlermeldung wuenschenswert */
+		rerror(parserrsym,"while parsing",tmptoksym(tok));
+/****************************/
 		obj = parserrsym;
 	}
 	if(weiter)
@@ -596,7 +641,7 @@ truc *pobj;
 		if(n == mGLOBAL) {
 			if(sflg == sCONSTANT) {
 				sptr = symptr(obj);
-				obj = sptr->bind;
+				obj = sptr->bind.t;
 			}
 		}
 		else if(n & mGLOBAL) {
@@ -834,6 +879,9 @@ truc *strom;
 		else if(fun == stringsym) {
 			fun = str_sym;
 		}
+        else if(fun == gf2nintsym) {
+            fun = gf2n_sym;
+        }
 		else {
 			rerror(voidsym,err_funame,fun);
 			return(parserrsym);
@@ -1032,7 +1080,7 @@ truc *strom;
 	PARSpush(body);
 	sptr = symptr(fun);
 	*FLAGPTR(sptr) = (vflg ? sVFUNCTION : sFUNCTION);
-	sptr->bind = mkfundef(numparms,numopts,numvars);
+	sptr->bind.t = mkfundef(numparms,numopts,numvars);
 	obj = fun;
 
   cleanup4:
@@ -1673,7 +1721,8 @@ truc *strom;
 
 	typ = mksym(SymBuf,&sflg);
 	if(sflg == sTYPESPEC) {
-	    if(typ == integsym || typ == boolsym || typ == charsym) {
+	    if(typ == integsym || typ == boolsym || typ == charsym
+           || typ == gf2nintsym) {
 		    obj = SYMbind(typ);
 	    }
 	    else if(typ == realsym) {
@@ -1738,7 +1787,7 @@ truc *strom;
 		PARSnpop(2);
 	    }
 	    else if(typ == filesym || typ == stacksym || typ == symbsym) {
-		obj = SYMbind(typ);
+            obj = SYMbind(typ);
 	    }
 	}
 	else if(sflg == sTYPEDEF) {	/* user defined type */
@@ -1956,6 +2005,7 @@ int endtok;
 	truc obj;
 	int tok;
 	int n = 0;
+    char *errmess;
 
 	tok = nexttok(strom,1);		/* ueberlese linke Klammer */
 	while(tok != endtok) {
@@ -1970,8 +2020,8 @@ int endtok;
 		if(tok == COMMATOK)
 			nexttok(strom,1);
 		else if(tok != endtok) {
-/*********** unterscheide RPARENTOK und RBRACETOK! ********/
-			rerror(parserrsym,err_0rparen,voidsym);
+            errmess = (endtok == RBRACETOK ? err_0brace : err_0rparen);
+			rerror(parserrsym,errmess,voidsym);
 			goto errexit;
 		}
 	}
@@ -2042,7 +2092,7 @@ truc typ;
 	sptr = symptr(typ);
 	if(*FLAGPTR(sptr) != sTYPEDEF)
 		goto errexit;
-	typ = sptr->bind;
+	typ = sptr->bind.t;
 	flg = Tflag(typ);
 	if(flg == fSYMBOL)
 		goto nochmal;
@@ -2288,6 +2338,9 @@ truc *endptr;
 			PARSnpop(n);
 			return(aERROR);
 		}
+        else if(obj == enddotsym) {
+            break;
+        }
 		else {
 			PARSpush(obj);
 			n++;
@@ -2360,7 +2413,7 @@ truc obj;
 {
 	PARSpush(zero);
 	PARSpush(obj);
-		obj = optbnode(minussym);
+	obj = optbnode(uminsym);
 	PARSnpop(2);
 	return(obj);
 }
@@ -2376,4 +2429,156 @@ truc *strom;
 	if(*strom == tstdin)
 		dumpinput();
 }
+/*--------------------------------------------------------------*/
+PRIVATE truc tmptoksym(tok)
+int tok;
+{
+    char *name;
+    switch(tok) {
+    case EOFTOK:
+        name = "end of input";
+        break;
+    case EOLTOK:
+        name = "end-of-line token";
+        break;
+    case LPARENTOK:
+        name = "left parenthesis (";
+        break;
+    case RPARENTOK:
+        name = "right parentesis )";
+        break;
+    case LBRACKTOK:
+        name = "left bracket [";
+        break;
+    case RBRACKTOK:
+        name = "right bracket ]";
+        break;
+    case LBRACETOK:
+        name = "left brace {";
+        break;
+    case RBRACETOK:
+        name = "right brace }";
+        break;
+    case BEGCOMMTOK:
+        name = "begin comment token";
+        break;
+    case ENDCOMMTOK:
+        name = "end comment token";
+        break;
+    case COMMATOK:
+        name = "comma token";
+        break;
+    case COLONTOK:
+        name = "colon token ':'";
+        break;
+    case SEMICOLTOK:
+        name = "semi colon token";
+        break;
+    case DOTTOK:
+        name = "dot token";
+        break;
+    case DOTDOTTOK:
+        name = "dot dot token '..'";
+        break;
+    case RECDOTTOK:
+        name = "record field separator";
+        break;
+    case DEREFTOK:
+        name = "dereferencing token '^'";
+        break;
+    case DOLLARTOK:
+        name = "dollar token";
+        break;
+    case HISTORYTOK:
+        name = "history token '!'";
+        break;
+    case QUESTIONTOK:
+        name = "question mark";
+        break;
+    case ASSIGNTOK:
+        name = "assignment token :=";
+        break;
+    case ORTOK:
+        name = "or token";
+        break;
+    case ANDTOK:
+        name = "and token";
+        break;
+    case NOTTOK:
+        name = "not token";
+        break;
+    case EQTOK:
+        name = "equal token";
+        break;
+    case NETOK:
+        name = "not equal token";
+        break;
+    case LTTOK:
+        name = "less than token";
+        break;
+    case LETOK:
+        name = "token <=";
+        break;
+    case GTTOK:
+        name = "greater than token";
+        break;
+    case GETOK:
+        name = "token >=";
+        break;
+    case PLUSTOK:
+        name = "plus token";
+        break;
+    case MINUSTOK:
+        name = "minus token";
+        break;
+    case TIMESTOK:
+        name = "times token '*'";
+        break;
+    case DIVIDETOK:
+        name = "divide token '/'";
+        break;
+    case DIVTOK:
+        name = "div token";
+        break;
+    case MODTOK:
+        name = "mod token";
+        break;
+    case UMINUSTOK:
+        name = "unitary minus token";
+        break;
+    case POWERTOK:
+        name = "power token **";
+        break;
+    case BOOLTOK:
+        name = "boolean value";
+        break;
+    case CHARTOK:
+        name = "character token";
+        break;
+    case INUMTOK:
+        name = "integer";
+        break;
+    case GF2NTOK:
+        name = "gf2n_int";
+        break;
+    case FLOATTOK:
+        name = "floating point number";
+        break;
+    case STRINGTOK:
+        name = "string token";
+        break;
+    case BSTRINGTOK:
+        name = "byte_string token";
+        break;
+    case SYMBOLTOK:
+        name = "symbol";
+        break;
+    case VECTORTOK:
+        name = "array";
+        break;
+    default:
+        name = "unidentified token";
+    }
+    return(scratch(name));
+}
 /********************************************************************/
