@@ -2,7 +2,7 @@
 /* file scanner.c
 
 ARIBAS interpreter for Arithmetic
-Copyright (C) 1996 O.Forster
+Copyright (C) 1996-2002 O.Forster
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ Address of the author
     Theresienstr. 39
     D-80333 Muenchen, Germany
 
-Email   forster@rz.mathematik.uni-muenchen.de
+Email   forster@mathematik.uni-muenchen.de
 */
 /****************************************************************/
 
@@ -84,7 +84,7 @@ PRIVATE int histinp _((ifun0 nextch, char *buf));
 PRIVATE int stringinp   _((ifun0 nextch, char *buf, ifun delimfun));
 PRIVATE int charinp _((ifun0 nextch, char *buf));
 PRIVATE int symbolchar  _((int ch));
-PRIVATE int isbasemark  _((int ch));
+PRIVATE int isbasemark  _((int ch, int mode));
 PRIVATE int isexpmark   _((int ch));
 PRIVATE int normdecstr  _((char *str));
 PRIVATE int digsequence  _((ifun0 nextch, char *buf, int maxlen));
@@ -137,9 +137,9 @@ PUBLIC void iniscan()
 
     SymBuf = StrBuf + strbufSize;
 
-    readlnsym = newsymsig("readln", sSBINARY,(truc)Sreadln, s_0u);
-    atoisym   = newsymsig("atoi",   sSBINARY,(truc)Satoi,   s_12);
-    atofsym   = newsymsig("atof",   sSBINARY,(truc)Satof,   s_12);
+    readlnsym = newsymsig("readln", sSBINARY,(wtruc)Sreadln, s_0u);
+    atoisym   = newsymsig("atoi",   sSBINARY,(wtruc)Satoi,   s_12);
+    atofsym   = newsymsig("atof",   sSBINARY,(wtruc)Satof,   s_12);
 
     Curop = nullsym;
 }
@@ -495,8 +495,16 @@ int skip;   /* if skip != 0, skips EOL */
     case '\015':    /* handle DOS line endings under UNiX */
         nextch();
         goto nochmal;
-                break;
+        break;
 #endif
+	case ZESC:
+		ch = nextch();
+		if(ch == '\001') {
+			res = Z1TOK;
+			break;
+		}
+		else
+			goto nochmal;
     case EOF:
         res = EOFTOK;
         break;
@@ -764,23 +772,24 @@ int ch;
 /*
 ** sets global variable curbase and global function checkdig
 ** to appropriate value
+** mode = 0 or mode = 2
 */
-PRIVATE int isbasemark(ch)
-int ch;
+PRIVATE int isbasemark(ch,mode)
+int ch, mode;
 {
     switch(ch) {
         case 'x':
         case 'X':
             checkdig = ishexdigit;
             return(curbase = 16);
-        case 'o':
-        case 'O':
-            checkdig = isoctdigit;
-            return(curbase = 8);
         case 'y':
         case 'Y':
             checkdig = isbindigit;
             return(curbase = 2);
+        case 'o':
+        case 'O':
+            checkdig = isoctdigit;
+            return(curbase = 8);
         default:
             return(0);
     }
@@ -968,11 +977,21 @@ int maxlen;
     str0 = str;
     ch = str[0];
     ch1 = peekchar(nextch);
-    if(ch == '0' && isbasemark(ch1)) {
+    if(ch == '0' && isbasemark(ch1,0)) {
         nextch();
         *str = nextch();
         return(intinp(nextch,str,maxlen));
     }
+#ifdef GF2NINTEGER
+    else if(ch == '2' && isbasemark(ch1,2)) {
+        nextch();
+        *str = nextch();
+        tok = intinp(nextch,str,maxlen);
+        if(tok == INUMTOK)
+            tok = GF2NTOK;
+        return(tok);
+    }
+#endif /* GF2NINTEGER */
     else if(!isdecdigit(ch))
         return(aERROR);
     curbase = 10;
@@ -1392,7 +1411,7 @@ char *mess;
 {
     char *st = "terminal input";
     char *sf = "loaded file";
-    word4 src;
+    wtruc src;
 
     if(inpsource == STRINGINPUT)
         strcopy(OutBuf,"error while reading from string");
@@ -1400,11 +1419,11 @@ char *mess;
         strcopy(OutBuf,"error in function readln");
     else {
         if(inpsource == TERMINALINP)
-        src = (word4)st;
+            src = (wtruc)st;
         else    /* FILEINPUT */
-        src = (word4)sf;
+            src = (wtruc)sf;
         s2form(OutBuf,
-        "error in line <= ~D of ~A",(word4)(cStream->lineno),src);
+        "error in line <= ~D of ~A",(wtruc)(cStream->lineno),src);
     }
     fprintline(tstderr,OutBuf);
     return(error(sym1,mess,sym2));
