@@ -2,7 +2,7 @@
 /* file storage.c
 
 ARIBAS interpreter for Arithmetic
-Copyright (C) 1996 O.Forster
+Copyright (C) 1996-2002 O.Forster
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ Address of the author
 	Theresienstr. 39
 	D-80333 Muenchen, Germany
 
-Email	forster@rz.mathematik.uni-muenchen.de
+Email	forster@mathematik.uni-muenchen.de
 */
 /****************************************************************/
 
@@ -37,6 +37,7 @@ Email	forster@rz.mathematik.uni-muenchen.de
 ** 1995-04-08: fixed bug with hugefloat
 ** 1997-04-13: newreflsym(), neworg (newintsym)
 ** 2000-12-28: removed some debugging code
+** 2002-04-19: mkgf2n, mk0gf2n
 */
 
 #include "common.h"
@@ -50,11 +51,11 @@ PUBLIC truc mksym	_((char *name, int *sflgptr));
 PUBLIC truc scratch	_((char *name));
 PUBLIC truc newselfsym	_((char *name, int flg));
 PUBLIC truc newreflsym	_((char *name, int flg));
-PUBLIC truc newintsym	_((char *name, int flg, truc bind));
+PUBLIC truc newintsym	_((char *name, int flg, wtruc bind));
 PUBLIC int  tokenvalue	_((truc op));
 PUBLIC truc newsym	_((char *name, int flg, truc bind));
-PUBLIC truc newsymsig	_((char *name, int flg, truc bind, int sig));
-PUBLIC truc new0symsig	_((char *name, int flg, truc bind, int sig));
+PUBLIC truc newsymsig	_((char *name, int flg, wtruc bind, int sig));
+PUBLIC truc new0symsig	_((char *name, int flg, wtruc bind, int sig));
 PUBLIC truc mkcopy	_((truc *x));
 PUBLIC truc mkarrcopy	_((truc *x));
 PUBLIC truc mkinum	_((long n));
@@ -63,7 +64,8 @@ PUBLIC truc mklocsym	_((int flg, unsigned u));
 PUBLIC truc mkfixnum	_((unsigned n));
 PUBLIC truc mksfixnum	_((int n));
 PUBLIC truc mkint	_((int sign, word2 *arr, int len));
-PUBLIC truc mk0big	_((int sign, word2 *arr, int len));
+PUBLIC truc mkgf2n  _((word2 *arr, int len));
+PUBLIC truc mk0gf2n _((word2 *arr, int len));
 PUBLIC truc mkfloat	_((int prec, numdata *nptr));
 PUBLIC truc fltzero	_((int prec));
 PUBLIC truc mk0float	_((numdata *nptr));
@@ -144,7 +146,7 @@ int mode;	/* mode != 0: allocate new space for name */
 	else
 		((struct symbol *)ptr)->name = name;
 
-	((struct symbol *)ptr)->bind = zero;	/* zur Sicherheit */
+	((struct symbol *)ptr)->bind.t = zero;	/* zur Sicherheit */
 	((struct symbol *)ptr)->cc.xx = 0;
 
 	return(ptr);
@@ -271,7 +273,7 @@ int flg;
 {
 	truc obj;
 
-	obj = newintsym(name,flg,voidsym);
+	obj = newintsym(name,flg,(wtruc)0);
 	SYMbind(obj) = obj;
 
 	return(obj);
@@ -284,7 +286,7 @@ int flg;
 PUBLIC truc newintsym(name,flg,bind)
 char *name;
 int flg;
-truc bind;
+wtruc bind;
 {
 	truc *ptr;
 	variant v;
@@ -298,7 +300,7 @@ truc bind;
 	*OFFSPTR(ptr) = (word2)wo;
 
 	((struct intsymbol *)ptr)->name = name;
-	((struct intsymbol *)ptr)->bind = bind;
+	((struct intsymbol *)ptr)->bind.w = bind;
 
 	v.xx = *ptr;
 	v.pp.b0 = fSYMBOL;
@@ -313,7 +315,7 @@ truc bind;
 PUBLIC truc new0symsig(name,flg,bind,sig)
 char *name;
 int flg;
-truc bind;
+wtruc bind;
 int sig;
 {
 	truc *ptr;
@@ -322,7 +324,7 @@ int sig;
 	ptr = mksymaux(flg,name,0);
 	v.xx = *ptr;
 	v.pp.b0 = fSYMBOL;
-	((struct symbol *)ptr)-> bind = bind;
+	((struct symbol *)ptr)-> bind.w = bind;
 	((struct symbol *)ptr)->cc.yy.ww = sig;
 
 	return(v.xx);
@@ -354,7 +356,7 @@ truc bind;
 
 	ptr = findsym(name,0);
 	*FLAGPTR(ptr) = flg;
-	((struct symbol *)ptr)->bind = bind;
+	((struct symbol *)ptr)->bind.t = bind;
 	v.xx = *ptr;
 	v.pp.b0 = fSYMBOL;
 
@@ -376,7 +378,7 @@ int flg;
 	*FLAGPTR(ptr) = flg;
 	v.xx = *ptr;
 	v.pp.b0 = fSYMBOL;
-	((struct symbol *)ptr)->bind = v.xx;
+	((struct symbol *)ptr)->bind.t = v.xx;
 
 	return(v.xx);
 }
@@ -387,7 +389,7 @@ int flg;
 PUBLIC truc newsymsig(name,flg,bind,sig)
 char *name;
 int flg;
-truc bind;
+wtruc bind;
 int sig;
 {
 	variant v;
@@ -395,7 +397,7 @@ int sig;
 
 	ptr = findsym(name,0);
 	*FLAGPTR(ptr) = flg;
-	((struct symbol *)ptr)->bind = bind;
+	((struct symbol *)ptr)->bind.w = bind;
 	((struct symbol *)ptr)->cc.yy.ww = sig;
 	v.xx = *ptr;
 	v.pp.b0 = fSYMBOL;
@@ -415,7 +417,7 @@ truc *x;
 	unsigned int len;
 	int flg = *FLAGPTR(x);
 
-	if(flg & FIXMASK)
+	if((flg & FIXMASK) || (!*SEGPTR(x)))
 		return(*x);
 	len = obj4size(flg,TAddress(x));
 	obj = newobj(flg,len,&ptr);
@@ -582,20 +584,44 @@ int len;
 }
 /*--------------------------------------------------------------*/
 /*
-** make bignum which is not moved during garbage collection
+** make gf2nint from (arr,len)
 */
-PUBLIC truc mk0big(sign,arr,len)
-int sign;
+PUBLIC truc mkgf2n(arr,len)
 word2 *arr;
-int len;	/* supposed to be >= 2 */
+int len;
+{
+	struct bigcell *big;
+	truc  obj;
+
+	if(len <= 1) {
+        if (!len)
+            return gf2nzero;
+        else if (arr[0] == 1)
+            return gf2none;
+	/* else fall through */
+    }
+	obj = newobj(fGF2NINT,SIZEOFBIG(len),(trucptr *)&big);
+	big->flag = fGF2NINT;
+	big->signum = 0;
+	big->len = len;
+	cpyarr(arr,len,&(big->digi0));
+	return(obj);
+}
+/*--------------------------------------------------------------*/
+/*
+** make gf2nint which is not moved during garbage collection
+*/
+PUBLIC truc mk0gf2n(arr,len)
+word2 *arr;
+int len;
 {
 	struct bigcell *big;
 	truc obj;
 
-	obj = new0obj(fBIGNUM,SIZEOFBIG(len),(trucptr *)&big);
+	obj = new0obj(fGF2NINT,SIZEOFBIG(len),(trucptr *)&big);
 
-	big->flag = fBIGNUM;
-	big->signum = sign;
+	big->flag = fGF2NINT;
+	big->signum = 0;
 	big->len = len;
 	cpyarr(arr,len,&(big->digi0));
 
